@@ -25,17 +25,11 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
-"""
-
-Utilities, mainly for image treatment
-
-"""
-
 __authors__ = ["Jérôme Kieffer", "Valentin Valls"]
 __contact__ = "Jerome.Kieffer@ESRF.eu"
 __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
-__date__ = "22/10/2018"
+__date__ = "08/01/2021"
 __status__ = "production"
 
 import logging
@@ -47,8 +41,9 @@ from scipy.interpolate import interp1d
 from scipy.optimize.optimize import fmin
 from scipy.optimize.optimize import fminbound
 
-from .third_party import six
 from .utils import stringutil
+from .utils import header_utils
+from .io.image import read_data
 
 from ._version import calc_hexversion
 if ("hexversion" not in dir(fabio)) or (fabio.hexversion < calc_hexversion(0, 4, 0, "dev", 5)):
@@ -177,6 +172,7 @@ class ImageStackFilter(ImageReductionFilter):
     Filter creating a stack from all images and computing everything at the
     end.
     """
+
     def init(self, max_images=None):
         self._stack = None
         self._max_stack_size = max_images
@@ -217,6 +213,7 @@ class AverageDarkFilter(ImageStackFilter):
 
     TODO: Must be split according to each filter_name, and removed
     """
+
     def __init__(self, filter_name, cut_off, quantiles):
         super(AverageDarkFilter, self).__init__()
         self._filter_name = filter_name
@@ -307,7 +304,7 @@ def bounding_box(img):
     Tries to guess the bounding box around a valid massif
 
     :param img: 2D array like
-    :return: 4-typle (d0_min, d1_min, d0_max, d1_max)
+    :return: 4-tuple (d0_min, d1_min, d0_max, d1_max)
     """
     img = img.astype(numpy.int)
     img0 = (img.sum(axis=1) > 0).astype(numpy.int)
@@ -327,12 +324,12 @@ def bounding_box(img):
 
 def remove_saturated_pixel(ds, threshold=0.1, minimum=None, maximum=None):
     """
-    Remove saturated fixes from an array inplace.
+    Remove saturated fixes from an array in place.
 
     :param ds: a dataset as ndarray
     :param float threshold: what is the upper limit?
-        all pixel > max*(1-threshold) are discareded.
-    :param float minimum: minumum valid value (or True for auto-guess)
+        all pixel > max*(1-threshold) are discarded.
+    :param float minimum: minimum valid value (or True for auto-guess)
     :param float maximum: maximum valid value
     :return: the input dataset
     """
@@ -355,7 +352,7 @@ def remove_saturated_pixel(ds, threshold=0.1, minimum=None, maximum=None):
     invalid = (ds > maxt)
     if minimum:
         if minimum is True:
-            # automatic guess of the best minimum TODO: use the HWHM to guess the minumum...
+            # automatic guess of the best minimum TODO: use the HWHM to guess the minimum...
             data_min = ds.min()
             x, y = numpy.histogram(numpy.log(ds - data_min + 1.0), bins=100)
             f = interp1d((y[1:] + y[:-1]) / 2.0, -x, bounds_error=False, fill_value=-x.min())
@@ -367,7 +364,7 @@ def remove_saturated_pixel(ds, threshold=0.1, minimum=None, maximum=None):
             else:
                 min_center = max_hi
             minimum = float(numpy.exp(y[((min_center / y) > 1).sum() - 1])) - 1.0 + data_min
-            logger.debug("removeSaturatedPixel: best minimum guessed is %s", minimum)
+            logger.debug("remove_saturated_pixel: best minimum guessed is %s", minimum)
         ds[ds < minimum] = minimum
         ds -= minimum  # - 1.0
 
@@ -393,7 +390,7 @@ def remove_saturated_pixel(ds, threshold=0.1, minimum=None, maximum=None):
 
 def average_dark(lstimg, center_method="mean", cutoff=None, quantiles=(0.5, 0.5)):
     """
-    Averages a serie of dark (or flat) images.
+    Averages a series of dark (or flat) images.
     Centers the result on the mean or the median ...
     but averages all frames within  cutoff*std
 
@@ -455,88 +452,6 @@ def average_dark(lstimg, center_method="mean", cutoff=None, quantiles=(0.5, 0.5)
     return output
 
 
-class MonitorNotFound(Exception):
-    """Raised when monitor information in not found or is not valid."""
-    pass
-
-
-def _get_monitor_value_from_edf(image, monitor_key):
-    """Return the monitor value from an EDF image using an header key.
-
-    Take care of the counter and motor syntax using for example 'counter/bmon'
-    which reach 'bmon' value from 'counter_pos' key using index from
-    'counter_mne' key.
-
-    :param fabio.fabioimage.FabioImage image: Image containing the header
-    :param str monitor_key: Key containing the monitor
-    :return: returns the monitor else raise a MonitorNotFound
-    :rtype: float
-    :raise MonitorNotFound: when the expected monitor is not found on the
-        header
-    """
-    keys = image.header
-
-    if "/" in monitor_key:
-        base_key, mnemonic = monitor_key.split('/', 1)
-
-        mnemonic_values_key = base_key + "_mne"
-        mnemonic_values = keys.get(mnemonic_values_key, None)
-        if mnemonic_values is None:
-            raise MonitorNotFound("Monitor mnemonic key '%s' not found in the header" % (mnemonic_values_key))
-
-        mnemonic_values = mnemonic_values.split()
-        pos_values_key = base_key + "_pos"
-        pos_values = keys.get(pos_values_key)
-        if pos_values is None:
-            raise MonitorNotFound("Monitor pos key '%s' not found in the header" % (pos_values_key))
-
-        pos_values = pos_values.split()
-
-        try:
-            index = mnemonic_values.index(mnemonic)
-        except ValueError:
-            logger.debug("Exception", exc_info=1)
-            raise MonitorNotFound("Monitor mnemonic '%s' not found in the header key '%s'" % (mnemonic, mnemonic_values_key))
-
-        if index >= len(pos_values):
-            raise MonitorNotFound("Monitor value '%s' not found in '%s'. Not enougth values." % (pos_values_key))
-
-        monitor = pos_values[index]
-
-    else:
-        if monitor_key not in keys:
-            raise MonitorNotFound("Monitor key '%s' not found in the header" % (monitor_key))
-        monitor = keys[monitor_key]
-
-    try:
-        monitor = float(monitor)
-    except ValueError as _e:
-        logger.debug("Exception", exc_info=1)
-        raise MonitorNotFound("Monitor value '%s' is not valid" % (monitor))
-    return monitor
-
-
-def get_monitor_value(image, monitor_key):
-    """Return the monitor value from an image using an header key.
-
-    :param fabio.fabioimage.FabioImage image: Image containing the header
-    :param str monitor_key: Key containing the monitor
-    :return: returns the monitor else raise an exception
-    :rtype: float
-    :raise MonitorNotFound: when the expected monitor is not found on the
-        header
-    """
-    if monitor_key is None:
-        return Exception("No monitor defined")
-
-    if isinstance(image, fabio.edfimage.EdfImage):
-        return _get_monitor_value_from_edf(image, monitor_key)
-    elif isinstance(image, fabio.numpyimage.numpyimage):
-        return _get_monitor_value_from_edf(image, monitor_key)
-    else:
-        raise Exception("File format '%s' unsupported" % type(image))
-
-
 def _normalize_image_stack(image_stack):
     """
     Convert input data to a list of 2D numpy arrays or a stack
@@ -558,8 +473,8 @@ def _normalize_image_stack(image_stack):
         # list of numpy images (multi 2D images)
         result = []
         for image in image_stack:
-            if isinstance(image, six.string_types):
-                data = fabio.open(image).data
+            if isinstance(image, (str,)):
+                data = read_data(image)
             elif isinstance(image, numpy.ndarray) and image.ndim == 2:
                 data = image
             else:
@@ -794,20 +709,26 @@ class Average(object):
         self._fabio_images = []
         self._nb_frames = 0
         if len(image_list) > 100:
-            # if too many files are opened, it may crash. The har limit is 1024
+            # if too many files are opened, it may crash. The hard limit is 1024
             copy_data = True
         else:
             copy_data = False
         for image_index, image in enumerate(image_list):
-            if isinstance(image, six.string_types):
+            if isinstance(image, (str,)):
                 logger.info("Reading %s", image)
-                fabio_image = fabio.open(image)
-                if copy_data and fabio_image.nframes == 1:
-                    # copy the data so that we can close the file right now.
-                    fimg = fabio_image.convert(fabio_image.__class__)
-                    fimg.filename = image
-                    fabio_image.close()
-                    fabio_image = fimg
+                try:
+                    fabio_image = fabio.open(image)
+                except:
+                    # Handles the different URL like data
+                    data = read_data(image)
+                    fabio_image = fabio.numpyimage.NumpyImage(data)
+                else:
+                    if copy_data and fabio_image.nframes == 1:
+                        # copy the data so that we can close the file right now.
+                        fimg = fabio_image.convert(fabio_image.__class__)
+                        fimg.filename = image
+                        fabio_image.close()
+                        fabio_image = fimg
             elif isinstance(image, fabio.fabioimage.fabioimage):
                 fabio_image = image
             else:
@@ -840,7 +761,7 @@ class Average(object):
         processing the average.
 
         :param threshold: what is the upper limit?
-            all pixel > max*(1-threshold) are discareded.
+            all pixel > max*(1-threshold) are discarded.
         :param minimum: minimum valid value or True
         :param maximum: maximum valid value
         """
@@ -880,9 +801,9 @@ class Average(object):
             corrected_image /= self._flat
         if self._monitor_key is not None:
             try:
-                monitor = get_monitor_value(fabio_image, self._monitor_key)
+                monitor = header_utils.get_monitor_value(fabio_image, self._monitor_key)
                 corrected_image /= monitor
-            except MonitorNotFound as e:
+            except header_utils.MonitorNotFound as e:
                 logger.warning("Monitor not found in filename '%s', data skipped. Cause: %s", fabio_image.filename, str(e))
                 return None
         return numpy.ascontiguousarray(corrected_image, numpy.float32)
@@ -985,7 +906,7 @@ def average_images(listImages, output=None, threshold=0.1, minimum=None,
     :param listImages: list of string representing the filenames
     :param output: name of the optional output file
     :param threshold: what is the upper limit? all pixel > max*(1-threshold)
-        are discareded.
+        are discarded.
     :param minimum: minimum valid value or True
     :param maximum: maximum valid value
     :param darks: list of dark current images for subtraction
