@@ -4,7 +4,7 @@
 #    Project: Azimuthal integration
 #             https://github.com/silx-kit/pyFAI
 #
-#    Copyright (C) 2013-2018 European Synchrotron Radiation Facility, Grenoble, France
+#    Copyright (C) 2013-2020 European Synchrotron Radiation Facility, Grenoble, France
 #
 #  Permission is hereby granted, free of charge, to any person obtaining a copy
 #  of this software and associated documentation files (the "Software"), to deal
@@ -24,13 +24,11 @@
 #  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 #  THE SOFTWARE.
 
-from __future__ import absolute_import, print_function, division, with_statement
-
 __author__ = "Jérôme Kieffer"
 __contact__ = "Jerome.Kieffer@ESRF.eu"
 __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
-__date__ = "03/12/2018"
+__date__ = "08/01/2021"
 __status__ = "development"
 
 import logging
@@ -46,7 +44,6 @@ if ocl:
     from .opencl import azim_csr as ocl_azim_csr
 else:
     ocl_azim_lut = ocl_azim_csr = None
-from .third_party import six
 
 try:
     from .ext import _distortion
@@ -57,7 +54,7 @@ except ImportError:
     _distortion = None
 
 try:
-    from scipy.sparse import linalg, csr_matrix
+    from scipy.sparse import linalg, csr_matrix, identity
 except IOError:
     logger.warning("Scipy is missing ... uncorrection will be handled the old way")
     linalg = None
@@ -69,14 +66,29 @@ else:
         linalg = None
 
 
+def resize_image_2D_numpy(image, shape_in):
+    "numpy implementation of resize_image_2D"
+    new_img = numpy.zeros(shape_in, dtype=image.dtype)
+    common_shape = [min(i, j) for i, j in zip(image.shape, shape_in)]
+    new_img[:common_shape[0],:common_shape[1]] = image[:common_shape[0],:common_shape[1]]
+    return new_img
+
+
+if _distortion is None:
+    resize_image_2D = resize_image_2D_numpy
+else:
+    from .ext._distortion import resize_image_2D
+
+
 class Distortion(object):
     """
     This class applies a distortion correction on an image.
 
     New version compatible both with CSR and LUT...
     """
+
     def __init__(self, detector="detector", shape=None, resize=False, empty=0,
-                 mask=None, method="CSR", device=None, workgroup=8):
+                 mask=None, method="CSR", device=None, workgroup=32):
         """
         :param detector: detector instance or detector name
         :param shape: shape of the output image
@@ -87,15 +99,17 @@ class Distortion(object):
         :param workgroup: workgroup size for CSR on OpenCL
         """
         self._shape_out = None
-        if isinstance(detector, six.string_types):
+        if isinstance(detector, (str,)):
             self.detector = detectors.detector_factory(detector)
         else:  # we assume it is a Detector instance
             self.detector = detector
         self.shape_in = self.detector.shape
-        if mask is not None:
-            self.mask = numpy.ascontiguousarray(mask, numpy.int8)
-        else:
+        if mask is None:
             self.mask = numpy.ascontiguousarray(self.detector.mask, numpy.int8)
+        elif mask is False:
+            self.mask = numpy.zeros(self.detector.shape, numpy.int8)
+        else:
+            self.mask = numpy.ascontiguousarray(mask, numpy.int8)
         self.resize = resize
         if shape is not None:
             self._shape_out = tuple([int(i) for i in shape])
@@ -109,18 +123,33 @@ class Distortion(object):
         self.bin_size = None
         self.max_size = None
         self.pos = None
-        self.lut = None
-        self.delta1 = self.delta2 = None  # max size of an pixel on a regular grid ...
-        self.offset1 = self.offset2 = 0  # position of the first bin
-        self.integrator = None
-        self.empty = empty  # "dummy" value for empty bins
         if not method:
             self.method = "lut"
         else:
             self.method = method.lower()
+        if (self.detector.uniform_pixel and self.detector.IS_FLAT):
+            sparse = identity(numpy.prod(self.detector.shape),
+                              dtype=numpy.float32,
+                              format="coo")
+            if self.detector.mask is not None:
+                masked = numpy.where(self.detector.mask.ravel())
+                sparse.data[masked] = 0.0
+                sparse.eliminate_zeros()
+            csr = sparse.tocsr()
+            if self.method == "lut":
+                self.lut = sparse_utils.CSR_to_LUT(csr.data, csr.indices, csr.indptr)
+            else:
+                self.lut = csr.data, csr.indices, csr.indptr
+        else:
+            # initialize the LUT later
+            self.lut = None
+        self.delta1 = self.delta2 = None  # max size of an pixel on a regular grid ...
+        self.offset1 = self.offset2 = 0  # position of the first bin
+        self.integrator = None
+        self.empty = empty  # "dummy" value for empty bins
         self.device = device
         if not workgroup:
-            self.workgroup = 8
+            self.workgroup = 1
         else:
             self.workgroup = int(workgroup)
 
@@ -170,6 +199,7 @@ class Distortion(object):
         :return: pixel corner positions (in pixel units) on the regular grid
         :rtype: ndarray of shape (nrow, ncol, 4, 2)
         """
+        logger.debug("in Distortion.calc_pos")
         if self.delta1 is None:
             with self._sem:
                 if self.delta1 is None:
@@ -209,21 +239,22 @@ class Distortion(object):
         [-17.48634 : 1027.0543, -22.768829 : 2028.3689]
         We chose to discard pixels falling outside the [0:1025,0:2048] range with a lose of intensity
         """
-        if self.pos is None:
-            pos = self.calc_pos()
-        else:
-            pos = self.pos
+        logger.debug("in Distortion.calc_size")
         if self.max_size is None:
+            if self.pos is None:
+                pos = self.calc_pos()
+            else:
+                pos = self.pos
             with self._sem:
                 if self.max_size is None:
                     if _distortion and use_cython:
                         self.bin_size = _distortion.calc_size(self.pos, self._shape_out, self.mask, (self.offset1, self.offset2))
                     else:
                         mask = self.mask
-                        pos0min = (numpy.floor(pos[:, :, :, 0].min(axis=-1) - self.offset1).astype(numpy.int32)).clip(0, self._shape_out[0])
-                        pos1min = (numpy.floor(pos[:, :, :, 1].min(axis=-1) - self.offset2).astype(numpy.int32)).clip(0, self._shape_out[1])
-                        pos0max = (numpy.ceil(pos[:, :, :, 0].max(axis=-1) - self.offset1 + 1).astype(numpy.int32)).clip(0, self._shape_out[0])
-                        pos1max = (numpy.ceil(pos[:, :, :, 1].max(axis=-1) - self.offset2 + 1).astype(numpy.int32)).clip(0, self._shape_out[1])
+                        pos0min = (numpy.floor(pos[:,:,:, 0].min(axis=-1) - self.offset1).astype(numpy.int32)).clip(0, self._shape_out[0])
+                        pos1min = (numpy.floor(pos[:,:,:, 1].min(axis=-1) - self.offset2).astype(numpy.int32)).clip(0, self._shape_out[1])
+                        pos0max = (numpy.ceil(pos[:,:,:, 0].max(axis=-1) - self.offset1 + 1).astype(numpy.int32)).clip(0, self._shape_out[0])
+                        pos1max = (numpy.ceil(pos[:,:,:, 1].max(axis=-1) - self.offset2 + 1).astype(numpy.int32)).clip(0, self._shape_out[1])
                         self.bin_size = numpy.zeros(self._shape_out, dtype=numpy.int32)
                         for i in range(self.shape_in[0]):
                             for j in range(self.shape_in[1]):
@@ -236,8 +267,7 @@ class Distortion(object):
     def calc_init(self):
         """Initialize all arrays
         """
-        self.calc_pos()
-        self.calc_size()
+        logger.debug("in Distortion.calc_init")
         self.calc_LUT()
         if ocl and self.device is not None:
             if "lower" in dir(self.device):
@@ -251,6 +281,7 @@ class Distortion(object):
                                                                       self._shape_out[0] * self._shape_out[1],
                                                                       devicetype=self.device,
                                                                       block_size=self.workgroup)
+                    self.integrator.workgroup_size["csr_integrate4"] = 1,
             else:
                 if self.method == "lut":
                     self.integrator = ocl_azim_lut.OCL_LUT_Integrator(self.lut,
@@ -262,18 +293,21 @@ class Distortion(object):
                                                                       self._shape_out[0] * self._shape_out[1],
                                                                       platformid=self.device[0], deviceid=self.device[1],
                                                                       block_size=self.workgroup)
+                    self.integrator.workgroup_size["csr_integrate4"] = 1,
 
     def calc_LUT(self, use_common=True):
         """Calculate the Look-up table
 
         :return: look up table either in CSR or LUT format depending on serl.method
         """
-        if self.pos is None:
-            self.calc_pos()
+        logger.debug("in Distortion.calc_LUT")
 
-        if self.max_size is None and not use_common:
-            self.calc_size()
         if self.lut is None:
+            if self.pos is None:
+                self.calc_pos()
+
+            if self.max_size is None and not use_common:
+                self.calc_size()
             with self._sem:
                 if self.lut is None:
                     mask = self.mask
@@ -287,8 +321,8 @@ class Distortion(object):
                                 self.lut = _distortion.calc_CSR(self.pos, self._shape_out, self.bin_size, max_pixel_size=(self.delta1, self.delta2))
                     else:
                         lut = numpy.recarray(shape=(self._shape_out[0], self._shape_out[1], self.max_size), dtype=[("idx", numpy.uint32), ("coef", numpy.float32)])
-                        lut[:, :, :].idx = 0
-                        lut[:, :, :].coef = 0.0
+                        lut[:,:,:].idx = 0
+                        lut[:,:,:].coef = 0.0
                         outMax = numpy.zeros(self._shape_out, dtype=numpy.uint32)
                         idx = 0
                         buffer_ = numpy.empty((self.delta1, self.delta2))
@@ -298,14 +332,14 @@ class Distortion(object):
                                 if (mask is not None) and mask[i, j]:
                                     continue
                                 # i,j, idx are indexes of the raw image uncorrected
-                                quad.reinit(*list(self.pos[i, j, :, :].ravel()))
+                                quad.reinit(*list(self.pos[i, j,:,:].ravel()))
                                 # print(self.pos[i, j, 0, :], self.pos[i, j, 1, :], self.pos[i, j, 2, :], self.pos[i, j, 3, :]
                                 try:
                                     quad.populate_box()
                                 except Exception as error:
                                     print("error in quad.populate_box of pixel %i, %i: %s" % (i, j, error))
                                     print("calc_area_vectorial", quad.calc_area_vectorial())
-                                    print(self.pos[i, j, 0, :], self.pos[i, j, 1, :], self.pos[i, j, 2, :], self.pos[i, j, 3, :])
+                                    print(self.pos[i, j, 0,:], self.pos[i, j, 1,:], self.pos[i, j, 2,:], self.pos[i, j, 3,:])
                                     print(quad)
                                     raise
                 #                box = quad.get_box()
@@ -340,14 +374,7 @@ class Distortion(object):
         :return: corrected 2D image
         """
         if image.ndim == 2:
-            if _distortion:
-                image = _distortion.resize_image_2D(image, self.shape_in)
-            else:
-                logger.error("The image shape (%s) is not the same as the detector (%s). Adapting shape ...", image.shape, self.shape_in)
-                new_img = numpy.zeros(self.shape_in, dtype=image.dtype)
-                common_shape = [min(i, j) for i, j in zip(image.shape, self.shape_in)]
-                new_img[:common_shape[0], :common_shape[1]] = image[:common_shape[0], :common_shape[1]]
-                image = new_img
+            image = resize_image_2D(image, self.shape_in)
         else:  # assume 2d+nchanel
             if _distortion:
                 image = _distortion.resize_image_3D(image, self.shape_in)
@@ -359,14 +386,14 @@ class Distortion(object):
                     new_image = numpy.zeros((shape_in0, shape_in1, nchan), dtype=numpy.float32)
                     if shape_img0 < shape_in0:
                         if shape_img1 < shape_in1:
-                            new_image[:shape_img0, :shape_img1, :] = image
+                            new_image[:shape_img0,:shape_img1,:] = image
                         else:
-                            new_image[:shape_img0, :, :] = image[:, :shape_in1, :]
+                            new_image[:shape_img0,:,:] = image[:,:shape_in1,:]
                     else:
                         if shape_img1 < shape_in1:
-                            new_image[:, :shape_img1, :] = image[:shape_in0, :, :]
+                            new_image[:,:shape_img1,:] = image[:shape_in0,:,:]
                         else:
-                            new_image[:, :, :] = image[:shape_in0, :shape_in1, :]
+                            new_image[:,:,:] = image[:shape_in0,:shape_in1,:]
                     logger.warning("Patching image of shape %ix%i on expected size of %ix%i",
                                    shape_img1, shape_img0, shape_in1, shape_in0)
                 image = new_image
@@ -405,6 +432,100 @@ class Distortion(object):
             raise
         return out
 
+    def correct_ng(self, image,
+                   variance=None,
+                   dark=None,
+                   flat=None,
+                   solidangle=None,
+                   polarization=None,
+                   dummy=None,
+                   delta_dummy=None,
+                   normalization_factor=1.0):
+        """
+        Correct an image based on the look-up table calculated ...
+        Like the integrate_ng it provides
+        * Dark current correction
+        * Normalisation with flatfield (or solid angle, polarization, absorption, ...)
+        * Error propagation
+
+        :param image: 2D-array with the image
+        :param variance: 2D-array with the associated image
+        :param dark: array with dark-current values
+        :param flat: array with values for a flat image
+        :param solidangle: solid-angle array
+        :param polarization: numpy array with 2D polarization corrections
+        :param dummy: value suggested for bad pixels
+        :param delta_dummy: precision of the dummy value
+        :param normalization_factor: multiply all normalization with this value
+        :return: corrected 2D image
+        """
+        assert image.ndim == 2
+        if variance is not None:
+            assert variance.shape == image.shape
+
+        if image.shape != self.shape_in:
+            logger.warning("The image shape %s is not the same as the detector %s", image.shape, self.shape_in)
+            image = resize_image_2D(image, self.shape_in)
+            if variance is not None:
+                variance = resize_image_2D(variance, self.shape_in)
+            if dark is not None:
+                dark = resize_image_2D(dark, self.shape_in)
+
+        if self.device:
+            if self.integrator is None:
+                self.calc_init()
+            res = self.integrator.integrate_ng(image,
+                                               variance=variance,
+                                               flat=flat,
+                                               dark=dark,
+                                               solidangle=solidangle,
+                                               polarization=polarization,
+                                               dummy=dummy,
+                                               delta_dummy=delta_dummy,
+                                               normalization_factor=normalization_factor,
+                                               out_merged=False
+                                               )
+            if variance is None:
+                if image.ndim == 2:
+                    out = res.intensity.reshape(self._shape_out)
+                else:
+                    out = res.intensity
+            else:
+                if image.ndim == 2:
+                    out = (res.intensity.reshape(self._shape_out), res.error.reshape(self._shape_out))
+                else:
+                    out = (res.intensity, res.error)
+        else:
+            if self.lut is None:
+                self.calc_LUT()
+            if _distortion is not None:
+                out = _distortion.correct(image, self.shape_in, self._shape_out, self.lut,
+                                          dummy=dummy or self.empty, delta_dummy=delta_dummy)
+            else:
+                if self.method == "lut":
+                    big = image.ravel().take(self.lut.idx) * self.lut.coef
+                    out = big.sum(axis=-1)
+                elif self.method == "csr":
+                    big = self.lut[0] * image.ravel().take(self.lut[1])
+                    indptr = self.lut[2]
+                    out = numpy.zeros(indptr.size - 1)
+                    for i in range(indptr.size - 1):
+                        out[i] = big[indptr[i]:indptr[i + 1]].sum()
+            try:
+                if image.ndim == 2:
+                    out.shape = self._shape_out
+                else:
+                    for ds in out:
+                        if ds.ndim == 2:
+                            ds.shape = self._shape_out
+                        else:
+                            ds.shape = self._shape_out + ds.shape[2:]
+
+            except ValueError as _err:
+                logger.error("Requested in_shape=%s out_shape=%s and ", self.shape_in, self.shape_out)
+                raise
+        return out
+
     def uncorrect(self, image, use_cython=False):
         """
         Take an image which has been corrected and transform it into it's raw (with loss of information)
@@ -456,6 +577,7 @@ class Quad(object):
     .. image:: ../img/quad_model.svg
         :alt: Modelization of the quad
     """
+
     def __init__(self, buffer):
         self.box = buffer
         self.A0 = self.A1 = None
@@ -487,7 +609,7 @@ class Quad(object):
         return self.box_size1
 
     def reinit(self, A0, A1, B0, B1, C0, C1, D0, D1):
-        self.box[:, :] = 0.0
+        self.box[:,:] = 0.0
         self.A0 = A0
         self.A1 = A1
         self.B0 = B0
@@ -513,7 +635,7 @@ class Quad(object):
         self.area = None
 
     def __repr__(self):
-        return os.linesep.join(["offset %i,%i size %i, %i" % (self.offset0, self.offset1, self.box_size0, self.box_size1), "box: %s" % self.box[:self.box_size0, :self.box_size1]])
+        return os.linesep.join(["offset %i,%i size %i, %i" % (self.offset0, self.offset1, self.box_size0, self.box_size1), "box: %s" % self.box[:self.box_size0,:self.box_size1]])
 
     def init_slope(self):
         if self.pAB is None:
@@ -577,6 +699,7 @@ class Quad(object):
         if self.area is None:
             self.area = numpy.cross([self.C0 - self.A0, self.C1 - self.A1], [self.D0 - self.B0, self.D1 - self.B1]) / 2.0
         return self.area
+
     calc_area = calc_area_vectorial
 
     def populate_box(self):
@@ -588,19 +711,19 @@ class Quad(object):
         self.integrateAB(self.C0, self.B0, self.calc_area_BC)
         if (self.box / self.calc_area()).min() < 0:
             print(self.box)
-            self.box[:, :] = 0
+            self.box[:,:] = 0
             print("AB")
             self.integrateAB(self.B0, self.A0, self.calc_area_AB)
             print(self.box)
-            self.box[:, :] = 0
+            self.box[:,:] = 0
             print("DA")
             self.integrateAB(self.A0, self.D0, self.calc_area_DA)
             print(self.box)
-            self.box[:, :] = 0
+            self.box[:,:] = 0
             print("CD")
             self.integrateAB(self.D0, self.C0, self.calc_area_CD)
             print(self.box)
-            self.box[:, :] = 0
+            self.box[:,:] = 0
             print("BC")
             self.integrateAB(self.C0, self.B0, self.calc_area_BC)
             print(self.box)
