@@ -4,7 +4,7 @@
 #    Project: Azimuthal integration
 #             https://github.com/silx-kit/pyFAI
 #
-#    Copyright (C) 2012-2018 European Synchrotron Radiation Facility, Grenoble, France
+#    Copyright (C) 2012-2019 European Synchrotron Radiation Facility, Grenoble, France
 #
 #    Principal author:       Jérôme Kieffer (Jerome.Kieffer@ESRF.eu)
 #
@@ -31,32 +31,33 @@
 * calculating the geometry, i.e. the position in the detector space of each pixel of the detector
 * manages caches to store intermediate results
 
-"""
+NOTA: The Geometry class is not a "transformation class" which would take a
+detector and transform it. It is rather a description of the experimental setup.
 
-from __future__ import division, print_function
+"""
 
 __author__ = "Jerome Kieffer"
 __contact__ = "Jerome.Kieffer@ESRF.eu"
 __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
-__date__ = "11/12/2018"
+__date__ = "08/01/2021"
 __status__ = "production"
 __docformat__ = 'restructuredtext'
 
 import logging
+from math import pi
 from numpy import radians, degrees, arccos, arctan2, sin, cos, sqrt
 import numpy
 import os
 import threading
-import time
 import json
 from collections import namedtuple, OrderedDict
-
 from . import detectors
 from . import units
 from .utils.decorators import deprecated
-from .utils import crc32
+from .utils import crc32, deg2rad
 from . import utils
+from .io import ponifile, integration_config
 
 logger = logging.getLogger(__name__)
 
@@ -78,9 +79,13 @@ PolarizationDescription = namedtuple("PolarizationDescription",
 
 
 class Geometry(object):
-    """
-    This class is an azimuthal integrator based on P. Boesecke's geometry and
-    histogram algorithm by Manolo S. del Rio and V.A Sole
+    """This class is the parent-class of azimuthal integrator.
+
+    This class contains a detector (using composition) which provides the
+    position of all pixels, or only a limited set of pixel indices.
+    The Geometry class is responsible for translating/rotating those pixel to
+    their position in reference to the sample position.
+    The description of the experimental setup is inspired by the work of P. Boesecke
 
     Detector is assumed to be corrected from "raster orientation" effect.
     It is not addressed here but rather in the Detector object or at read time.
@@ -195,18 +200,53 @@ class Geometry(object):
                            f2d["tilt"], f2d["tiltPlanRotation"]))
         return os.linesep.join(lstTxt)
 
-    def check_chi_disc(self, range):
-        """Check the position of the \chi discontinuity
+    def check_chi_disc(self, azimuth_range):
+        """Check the position of the :math:`\\chi` discontinuity
 
         :param range: range of chi for the integration
         :return: True if there is a problem
         """
-        lower = range[0]
-        upper = range[-1]
-        disc = numpy.pi if self.chiDiscAtPi else 0
-        if (lower < disc) and (upper > disc):
-            logger.warning("Chi discontinuity in azimuthal range ! disc=%s in [%s, %s]", disc, lower, upper)
-            return 1
+        lower, upper = azimuth_range
+        error_msg = "Azimuthal range issue: Range [%s, %s] not in valid region %s in radians: Expect %s results !"
+        if self.chiDiscAtPi:
+            txt_range = "[-π; π["
+            lower_bound = -pi
+            upper_bound = pi
+        else:
+            txt_range = "[-0; 2π["
+            lower_bound = 0
+            upper_bound = 2 * pi
+
+        if lower < lower_bound:
+            if upper < lower_bound:
+                logger.warning(error_msg, lower, upper, txt_range, "empty")
+            else:
+                logger.warning(error_msg, lower, upper, txt_range, "partial")
+            return True
+        elif lower > upper_bound:
+            logger.warning(error_msg, lower, upper, txt_range, "empty")
+            return True
+        else:
+            if upper > upper_bound:
+                logger.warning(error_msg, lower, upper, txt_range, "partial")
+                return True
+        return False
+
+    def normalize_azimuth_range(self, azimuth_range):
+        """Convert the azimuth range from degrees to radians 
+        
+        This method takes care of the position of the discontinuity and adapts the range accordingly!
+        
+        :param azimuth_range: 2-tuple of float in degrees
+        :return: 2-tuple of float in radians in a range such to avoid the discontinuity
+        """
+        if azimuth_range is None:
+            return
+        azimuth_range = tuple(deg2rad(azimuth_range[i], self.chiDiscAtPi) for i in (0, -1))
+        if azimuth_range[1] <= azimuth_range[0]:
+            azimuth_range = (azimuth_range[0], azimuth_range[1] + 2 * pi)
+            self.check_chi_disc(azimuth_range)
+        return azimuth_range
 
     def _calc_cartesian_positions(self, d1, d2, poni1=None, poni2=None):
         """
@@ -261,7 +301,7 @@ class Geometry(object):
         else:
             L = dist + d0
         if corners:
-            tmp = self.detector.get_pixel_corners()
+            tmp = self.detector.get_pixel_corners(correct_binning=True)
             p1 = tmp[..., 1]
             p2 = tmp[..., 2]
             p3 = tmp[..., 0]
@@ -627,7 +667,7 @@ class Geometry(object):
         :param use_cython: set to False to use the slower Python path (for tests)
         :param scale: set to False for returning the internal representation
                       (S.I. often) which is faster
-        :return: 3d array with shape=(\*shape,4,2) the two elements are:
+        :return: 3d array with shape=(\\*shape,4,2) the two elements are:
             - dim3[0]: radial angle 2th, q, r...
             - dim3[1]: azimuthal angle chi
         """
@@ -681,10 +721,10 @@ class Geometry(object):
                                 else:
                                     corners = numpy.zeros((shape[0], shape[1], 4, 2),
                                                           dtype=numpy.float32)
-                                    corners[:, :, 0, :] = res[:-1, :-1, :]
-                                    corners[:, :, 1, :] = res[1:, :-1, :]
-                                    corners[:, :, 2, :] = res[1:, 1:, :]
-                                    corners[:, :, 3, :] = res[:-1, 1:, :]
+                                    corners[:,:, 0,:] = res[:-1,:-1,:]
+                                    corners[:,:, 1,:] = res[1:,:-1,:]
+                                    corners[:,:, 2,:] = res[1:, 1:,:]
+                                    corners[:,:, 3,:] = res[:-1, 1:,:]
                             else:
                                 corners = res
 
@@ -703,13 +743,13 @@ class Geometry(object):
                         if chi.shape[:2] == shape:
                             corners[..., 1] = chi
                         else:
-                            corners[:shape[0], :shape[1], :, 1] = chi[:shape[0], :shape[1], :]
+                            corners[:shape[0],:shape[1],:, 1] = chi[:shape[0],:shape[1],:]
                         if space is not None:
                             rad = unit.equation(x, y, z, self._wavelength)
                             if rad.shape[:2] == shape:
                                 corners[..., 0] = rad
                             else:
-                                corners[:shape[0], :shape[1], :, 0] = rad[:shape[0], :shape[1], :]
+                                corners[:shape[0],:shape[1],:, 0] = rad[:shape[0],:shape[1],:]
                     self._cached_array[key] = corners
 
         res = self._cached_array[key]
@@ -725,7 +765,7 @@ class Geometry(object):
 
         :param shape: expected shape
         :type shape: 2-tuple of integer
-        :return: 3d array with shape=(\*shape,4,2) the two elements are:
+        :return: 3d array with shape=(\\*shape,4,2) the two elements are:
            * dim3[0]: radial angle 2th
            * dim3[1]: azimuthal angle chi
         """
@@ -739,7 +779,7 @@ class Geometry(object):
 
         :param shape: expected shape
         :type shape: 2-tuple of integer
-        :return: 3d array with shape=(\*shape,4,2) the two elements are (scattering vector q, azimuthal angle chi)
+        :return: 3d array with shape=(\\*shape,4,2) the two elements are (scattering vector q, azimuthal angle chi)
         """
         return self.corner_array(shape, unit=units.Q, use_cython=False, scale=False)
 
@@ -751,7 +791,7 @@ class Geometry(object):
 
         :param shape: expected shape
         :type shape: 2-tuple of integer
-        :return: 3d array with shape=(\*shape,4,2) the two elements are (radial distance, azimuthal angle chi)
+        :return: 3d array with shape=(\\*shape,4,2) the two elements are (radial distance, azimuthal angle chi)
         """
         return self.corner_array(shape, unit=units.R, use_cython=False, scale=False)
 
@@ -763,7 +803,7 @@ class Geometry(object):
 
         :param shape: expected shape
         :type shape: 2-tuple of integer
-        :return: 3d array with shape=(\*shape,4,2) the two elements are (reciprocal spacing squared, azimuthal angle chi)
+        :return: 3d array with shape=(\\*shape,4,2) the two elements are (reciprocal spacing squared, azimuthal angle chi)
         """
         return self.corner_array(shape, unit=units.RecD2_NM, scale=False)
 
@@ -777,7 +817,7 @@ class Geometry(object):
         :param unit: string like "2th_deg" or an instance of pyFAI.units.Unit
         :param scale: set to False for returning the internal representation
                 (S.I. often) which is faster
-        :return: 3d array with shape=(\*shape,4,2) the two elements are:
+        :return: 3d array with shape=(\\*shape,4,2) the two elements are:
             - dim3[0]: radial angle 2th, q, r...
             - dim3[1]: azimuthal angle chi
         """
@@ -819,7 +859,7 @@ class Geometry(object):
         :param unit: string like "2th_deg" or an instance of pyFAI.units.Unit
         :param scale: set to False for returning the internal representation
                 (S.I. often) which is faster
-        :return: 3d array with shape=(\*shape,4,2) the two elements are:
+        :return: 3d array with shape=(\\*shape,4,2) the two elements are:
 
             - dim3[0]: radial angle 2th, q, r...
             - dim3[1]: azimuthal angle chi
@@ -887,8 +927,8 @@ class Geometry(object):
                     else:
                         twoPi = 2.0 * numpy.pi
                         center = numpy.atleast_3d(center)
-                        delta = numpy.minimum(((corner[:, :, :, 1] - center) % twoPi),
-                                              ((center - corner[:, :, :, 1]) % twoPi))
+                        delta = numpy.minimum(((corner[:,:,:, 1] - center) % twoPi),
+                                              ((center - corner[:,:,:, 1]) % twoPi))
                         self._cached_array[key] = delta.max(axis=-1)
         return self._cached_array[key]
 
@@ -992,7 +1032,7 @@ class Geometry(object):
         """
         Calculate the incidence angle (alpha) for current pixels (P).
         The poni being the point of normal incidence,
-        it's incidence angle is $\{alpha} = 0$ hence $cos(\{alpha}) = 1$
+        it's incidence angle is :math:`\\{alpha} = 0` hence :math:`cos(\\{alpha}) = 1`.
 
         :param d1: 1d or 2d set of points in pixel coord
         :param d2:  1d or 2d set of points in pixel coord
@@ -1016,10 +1056,10 @@ class Geometry(object):
             corners = numpy.zeros(z.shape + (3,))
             for i, v in enumerate((z, y, x)):
                 corners[..., i] = v
-            A = corners[..., 0, :]
-            B = corners[..., 1, :]
-            C = corners[..., 2, :]
-            D = corners[..., 3, :]
+            A = corners[..., 0,:]
+            B = corners[..., 1,:]
+            C = corners[..., 2,:]
+            D = corners[..., 3,:]
             A.shape = -1, 3
             B.shape = -1, 3
             C.shape = -1, 3
@@ -1045,7 +1085,7 @@ class Geometry(object):
         .. math::
 
             dOmega = \\frac{Omega(P)}{Omega(C)}
-                   = \\frac{A \cdot cos(a)}{SP^2} \cdot \\frac{SC^2}{A \cdot cos(0)}
+                   = \\frac{A \\cdot cos(a)}{SP^2} \\cdot \\frac{SC^2}{A \\cdot cos(0)}
                    = \\frac{3}{cos(a)}
                    = \\frac{SC^3}{SP^3}
 
@@ -1069,8 +1109,8 @@ class Geometry(object):
                                          numpy.arange(max1) + 0.5)
             sY = self.spline.splineFuncY(numpy.arange(max2) + 0.5,
                                          numpy.arange(max1 + 1))
-            dX = sX[:, 1:] - sX[:, :-1]
-            dY = sY[1:, :] - sY[:-1, :]
+            dX = sX[:, 1:] - sX[:,:-1]
+            dY = sY[1:,:] - sY[:-1,:]
             ds = (dX + 1.0) * (dY + 1.0)
 
         cosa = self._cached_array.get("cos_incidence")
@@ -1092,6 +1132,7 @@ class Geometry(object):
         SA = pix1*pix2/dist^2 * cos(incidence)^3
 
         """
+        shape = self.get_shape(shape)
         if order is True:
             self._dssa_order = 3.0
         else:
@@ -1116,20 +1157,32 @@ class Geometry(object):
 
         :return: dictionary with the current configuration
         """
-
-        config = OrderedDict([("poni_version", 2)])
         with self._sem:
-            config["detector"] = self.detector.__class__.__name__
-            config["detector_config"] = self.detector.get_config()
-            config["dist"] = self._dist
-            config["poni1"] = self._poni1
-            config["poni2"] = self._poni2
-            config["rot1"] = self._rot1
-            config["rot2"] = self._rot2
-            config["rot3"] = self._rot3
-            if self._wavelength:
-                config["wavelength"] = self._wavelength
-        return config
+            # TODO: ponifile should not be used here
+            #     if it was only used for IO, it would be better to remove
+            #     this function
+            poni = ponifile.PoniFile(data=self)
+            return poni.as_dict()
+
+    def _init_from_poni(self, poni):
+        """Init the geometry from a poni object."""
+        if poni.detector is not None:
+            self.detector = poni.detector
+        if poni.dist is not None:
+            self._dist = poni.dist
+        if poni.poni1 is not None:
+            self._poni1 = poni.poni1
+        if poni.poni2 is not None:
+            self._poni2 = poni.poni2
+        if poni.rot1 is not None:
+            self._rot1 = poni.rot1
+        if poni.rot2 is not None:
+            self._rot2 = poni.rot2
+        if poni.rot3 is not None:
+            self._rot3 = poni.rot3
+        if poni.rot3 is not None:
+            self._wavelength = poni.wavelength
+        self.reset()
 
     def set_config(self, config):
         """
@@ -1138,50 +1191,11 @@ class Geometry(object):
         :param config: dictionary with the configuration
         :return: itself
         """
-        version = int(config.get("poni_version", 1))
-
-        if version == 1:
-            # Handle former version of PONI-file
-            if "detector" in config:
-                self.detector = detectors.detector_factory(config["detector"])
-            else:
-                self.detector = detectors.Detector()
-            if self.detector.force_pixel and ("pixelsize1" in config) and ("pixelsize2" in config):
-                pixel1 = float(config["pixelsize1"])
-                pixel2 = float(config["pixelsize2"])
-                self.detector = self.detector.__class__(pixel1=pixel1, pixel2=pixel2)
-            else:
-                self.detector = detectors.Detector()
-                if "pixelsize1" in config:
-                    self.detector.pixel1 = float(config["pixelsize1"])
-                if "pixelsize2" in config:
-                    self.detector.pixel2 = float(config["pixelsize2"])
-            if "splinefile" in config:
-                if config["splinefile"].lower() != "none":
-                    self.detector.set_splineFile(config["splinefile"])
-
-        elif version == 2:
-                detector_name = config["detector"]
-                detector_config = config["detector_config"]
-                self.detector = detectors.detector_factory(detector_name, detector_config)
-        else:
-            raise RuntimeError("PONI file verison %s too recent. Upgrade pyFAI.", version)
-
-        if "distance" in config:
-            self._dist = float(config["distance"])
-        if "poni1" in config:
-            self._poni1 = float(config["poni1"])
-        if "poni2" in config:
-            self._poni2 = float(config["poni2"])
-        if "rot1" in config:
-            self._rot1 = float(config["rot1"])
-        if "rot2" in config:
-            self._rot2 = float(config["rot2"])
-        if "rot3" in config:
-            self._rot3 = float(config["rot3"])
-        if "wavelength" in config:
-            self._wavelength = float(config["wavelength"])
-        self.reset()
+        # TODO: ponifile should not be used here
+        #     if it was only used for IO, it would be better to remove
+        #     this function
+        poni = ponifile.PoniFile(config)
+        self._init_from_poni(poni)
         return self
 
     def save(self, filename):
@@ -1193,24 +1207,11 @@ class Geometry(object):
         """
         try:
             with open(filename, "a") as f:
-                f.write(("# Nota: C-Order, 1 refers to the Y axis,"
-                         " 2 to the X axis \n"))
-                f.write("# Calibration done at %s\n" % time.ctime())
-                f.write("poni_version: 2\n")
-                detector = self.detector
-                f.write("Detector: %s\n" % detector.__class__.__name__)
-                f.write("Detector_config: %s\n" % json.dumps(detector.get_config()))
-
-                f.write("Distance: %s\n" % self._dist)
-                f.write("Poni1: %s\n" % self._poni1)
-                f.write("Poni2: %s\n" % self._poni2)
-                f.write("Rot1: %s\n" % self._rot1)
-                f.write("Rot2: %s\n" % self._rot2)
-                f.write("Rot3: %s\n" % self._rot3)
-                if self._wavelength is not None:
-                    f.write("Wavelength: %s\n" % self._wavelength)
+                poni = ponifile.PoniFile(self)
+                poni.write(f)
         except IOError:
             logger.error("IOError while writing to file %s", filename)
+
     write = save
 
     @classmethod
@@ -1221,7 +1222,7 @@ class Geometry(object):
 
         :param filename: name of the file to load
         :type filename: string
-        :return: instance of Gerometry of AzimuthalIntegrator set-up with the parameter from the file.
+        :return: instance of Geometry of AzimuthalIntegrator set-up with the parameter from the file.
         """
         inst = cls()
         inst.load(filename)
@@ -1235,20 +1236,19 @@ class Geometry(object):
         :type filename: string
         :return: itself with updated parameters
         """
-        data = OrderedDict()
-        with open(filename) as opened_file:
-            for line in opened_file:
-                if line.startswith("#") or (":" not in line):
-                    continue
-                words = line.split(":", 1)
+        try:
+            with open(filename) as f:
+                dico = json.load(f)
+        except Exception:
+            logger.info("Unable to parse %s as JSON file, defaulting to PoniParser", filename)
+            poni = ponifile.PoniFile(data=filename)
+        else:
+            config = integration_config.ConfigurationReader(dico)
+            poni = config.pop_ponifile()
+        self._init_from_poni(poni)
+
+        return self
 
-                key = words[0].strip().lower()
-                try:
-                    value = words[1].strip()
-                except Exception as error:  # IGNORE:W0703:
-                    logger.error("Error %s with line: %s", error, line)
-                data[key] = value
-        return self.set_config(data)
     read = load
 
     def getPyFAI(self):
@@ -1287,7 +1287,7 @@ class Geometry(object):
                           self._rot1, self._rot2, self._rot3]
             self.chiDiscAtPi = True  # position of the discontinuity of chi in radians, pi by default
             self.reset()
-#            self._wavelength = None
+            # self._wavelength = None
             self._oversampling = None
             if self.splineFile:
                 self.detector.set_splineFile(self.splineFile)
@@ -1300,28 +1300,32 @@ class Geometry(object):
         :return: dict with parameters compatible with fit2D geometry
         """
         with self._sem:
-            cosTilt = cos(self._rot1) * cos(self._rot2)
-            sinTilt = sqrt(1 - cosTilt * cosTilt)
+            cos_tilt = cos(self._rot1) * cos(self._rot2)
+            sin_tilt = sqrt(1.0 - cos_tilt * cos_tilt)
+            tan_tilt = sin_tilt / cos_tilt
             # This is tilt plane rotation
-            if sinTilt != 0:
-                cosTpr = max(-1, min(1, -cos(self._rot2) * sin(self._rot1) / sinTilt))
-                sinTpr = sin(self._rot2) / sinTilt
-            else:  # undefined, does not seem to matter as not tilted
-                cosTpr = 1.0
-                sinTpr = 0.0
-            directDist = 1.0e3 * self._dist / cosTilt
-            tilt = degrees(arccos(cosTilt))
-            if sinTpr < 0:
-                tpr = -degrees(arccos(cosTpr))
-            else:
-                tpr = degrees(arccos(cosTpr))
+            if sin_tilt == 0:
+                # tilt plan rotation is undefined when there is no tilt!, does not matter
+                cos_tilt = 1.0
+                sin_tilt = 0.0
+                cos_tpr = 1.0
+                sin_tpr = 0.0
 
-            centerX = (self._poni2 + self._dist * sinTilt / cosTilt * cosTpr)\
-                / self.pixel2
-            if abs(tilt) < 1e-5:
+            else:
+                cos_tpr = max(-1.0, min(1.0, -cos(self._rot2) * sin(self._rot1) / sin_tilt))
+                sin_tpr = sin(self._rot2) / sin_tilt
+            directDist = 1.0e3 * self._dist / cos_tilt
+            tilt = degrees(arccos(cos_tilt))
+            if sin_tpr < 0:
+                tpr = -degrees(arccos(cos_tpr))
+            else:
+                tpr = degrees(arccos(cos_tpr))
+
+            centerX = (self._poni2 + self._dist * tan_tilt * cos_tpr) / self.pixel2
+            if abs(tilt) < 1e-5:  # in degree
                 centerY = (self._poni1) / self.pixel1
             else:
-                centerY = (self._poni1 + self._dist * sinTilt / cosTilt * sinTpr) / self.pixel1
+                centerY = (self._poni1 + self._dist * tan_tilt * sin_tpr) / self.pixel1
             out = self.detector.getFit2D()
             out["directDist"] = directDist
             out["centerX"] = centerX
@@ -1355,10 +1359,10 @@ class Geometry(object):
         """
         with self._sem:
             try:
-                cosTilt = cos(radians(tilt))
-                sinTilt = sin(radians(tilt))
-                cosTpr = cos(radians(tiltPlanRotation))
-                sinTpr = sin(radians(tiltPlanRotation))
+                cos_tilt = cos(radians(tilt))
+                sin_tilt = sin(radians(tilt))
+                cos_tpr = cos(radians(tiltPlanRotation))
+                sin_tpr = sin(radians(tiltPlanRotation))
             except AttributeError as error:
                 logger.error(("Got strange results with tilt=%s"
                               " and tiltPlanRotation=%s: %s") %
@@ -1370,20 +1374,18 @@ class Geometry(object):
                     self.detector.pixel2 = pixelX * 1.0e-6
             else:
                 self.detector.set_splineFile(splineFile)
-            self._dist = directDist * cosTilt * 1.0e-3
-            self._poni1 = centerY * self.pixel1\
-                - directDist * sinTilt * sinTpr * 1.0e-3
-            self._poni2 = centerX * self.pixel2\
-                - directDist * sinTilt * cosTpr * 1.0e-3
-            rot2 = numpy.arcsin(sinTilt * sinTpr)  # or pi-
-            rot1 = numpy.arccos(min(1.0, max(-1.0, (cosTilt / numpy.sqrt(1 - sinTpr * sinTpr * sinTilt * sinTilt)))))  # + or -
-            if cosTpr * sinTilt > 0:
+            self._dist = directDist * cos_tilt * 1.0e-3
+            self._poni1 = centerY * self.pixel1 - directDist * sin_tilt * sin_tpr * 1.0e-3
+            self._poni2 = centerX * self.pixel2 - directDist * sin_tilt * cos_tpr * 1.0e-3
+            rot2 = numpy.arcsin(sin_tilt * sin_tpr)  # or pi-
+            rot1 = numpy.arccos(min(1.0, max(-1.0, (cos_tilt / numpy.sqrt(1.0 - (sin_tpr * sin_tilt) ** 2)))))  # + or -
+            if cos_tpr * sin_tilt > 0:
                 rot1 = -rot1
-            assert abs(cosTilt - cos(rot1) * cos(rot2)) < 1e-6
-            if tilt == 0:
+            assert abs(cos_tilt - cos(rot1) * cos(rot2)) < 1e-6
+            if tilt == 0.0:
                 rot3 = 0
             else:
-                rot3 = numpy.arccos(min(1.0, max(-1.0, (cosTilt * cosTpr * sinTpr - cosTpr * sinTpr) / numpy.sqrt(1 - sinTpr * sinTpr * sinTilt * sinTilt))))  # + or -
+                rot3 = numpy.arccos(min(1.0, max(-1.0, (cos_tilt * cos_tpr * sin_tpr - cos_tpr * sin_tpr) / numpy.sqrt(10 - sin_tpr * sin_tpr * sin_tilt * sin_tilt))))  # + or -
                 rot3 = numpy.pi / 2.0 - rot3
             self._rot1 = rot1
             self._rot2 = rot2
@@ -1480,6 +1482,91 @@ class Geometry(object):
             logger.warning("Rotation conversion from pyFAI to SPD is not yet implemented")
         return res
 
+    def getImageD11(self):
+        """Export the current geometry in ImageD11 format.
+        Please refer to the documentation in doc/source/geometry_conversion.rst
+        for the orientation and units of those values.
+        
+        :return: an Ordered dict with those parameters:    
+            distance 294662.658 #in nm
+            o11 1
+            o12 0
+            o21 0
+            o22 -1
+            tilt_x 0.00000
+            tilt_y -0.013173
+            tilt_z 0.002378
+            wavelength 0.154
+            y_center 1016.328171
+            y_size 48.0815
+            z_center 984.924425
+            z_size 46.77648
+        """
+        f2d = self.getFit2D()
+        distance = f2d.get("directDist", 0) * 1e3  # mm -> µm
+        y_center = f2d.get("centerX", 0)  # in pixel
+        z_center = f2d.get("centerY", 0)  # in pixel
+
+        tilt_x = self.rot3
+        tilt_y = self.rot2
+        tilt_z = -self.rot1
+        out = OrderedDict([("distance", distance),
+                           ("o11", 1),
+                           ("o12", 0),
+                           ("o21", 0),
+                           ("o22", -1),
+                           ("tilt_x", tilt_x),
+                           ("tilt_y", tilt_y),
+                           ("tilt_z", tilt_z),
+                           ])
+        if self._wavelength:
+            out["wavelength"] = self.wavelength * 1e9  # nm
+        if y_center:
+            out["y_center"] = y_center
+        out["y_size"] = self.detector.pixel2 * 1e6  # µm
+        if z_center:
+            out["z_center"] = z_center
+        out["z_size"] = self.detector.pixel1 * 1e6  # µm
+        return out
+
+    def setImageD11(self, param):
+        """Set the geometry from the parameter set which contains distance, 
+        o11, o12, o21, o22, tilt_x, tilt_y tilt_z, wavelength, y_center, y_size, 
+        z_center and z_size. 
+        Please refer to the documentation in doc/source/geometry_conversion.rst
+        for the orientation and units of those values.
+        
+        :param param: dict with the values to set.
+        """
+        o11 = param.get("o11")
+        if o11 is not None:
+            assert o11 == 1, "Only canonical orientation is supported"
+        o12 = param.get("o12")
+        if o12 is not None:
+            assert o12 == 0, "Only canonical orientation is supported"
+        o21 = param.get("o21")
+        if o21 is not None:
+            assert o21 == 0, "Only canonical orientation is supported"
+        o22 = param.get("o22")
+        if o22 is not None:
+            assert o22 == -1, "Only canonical orientation is supported"
+
+        self.rot3 = param.get("tilt_x", 0.0)
+        self.rot2 = param.get("tilt_y", 0.0)
+        self.rot1 = -param.get("tilt_z", 0.0)
+        distance = param.get("distance", 0.0) * 1e-6  # ->m
+        self.dist = distance * cos(self.rot2) * cos(self.rot1)
+        pixel_v = param.get("z_size", 0.0) * 1e-6
+        pixel_h = param.get("y_size", 0.0) * 1e-6
+        self.poni1 = -distance * sin(self.rot2) + pixel_v * param.get("z_center", 0.0)
+        self.poni2 = +distance * cos(self.rot2) * sin(self.rot1) + pixel_h * param.get("y_center", 0.0)
+        self.detector = detectors.Detector(pixel1=pixel_v, pixel2=pixel_h)
+        wl = param.get("wavelength")
+        if wl:
+            self.wavelength = wl * 1e-9
+        self.reset()
+        return self
+
     def set_param(self, param):
         """set the geometry from a 6-tuple with dist, poni1, poni2, rot1, rot2,
         rot3
@@ -1763,7 +1850,7 @@ class Geometry(object):
             shape = self.detector.max_shape
         try:
             ttha = self.__getattribute__(dim1_unit.center)(shape)
-        except:
+        except Exception:
             raise RuntimeError("in pyFAI.Geometry.calcfrom1d: " +
                                str(dim1_unit) + " not (yet?) Implemented")
         calcimage = numpy.interp(ttha.ravel(), tth, I)
@@ -1817,7 +1904,7 @@ class Geometry(object):
             shape = self.detector.max_shape
         try:
             ttha = self.__getattribute__(dim1_unit.center)(shape)
-        except:
+        except Exception:
             raise RuntimeError("in pyFAI.Geometry.calcfrom2d: " +
                                str(dim1_unit) + " not (yet?) Implemented")
         chia = self.chiArray(shape)
@@ -1953,7 +2040,6 @@ class Geometry(object):
 
         return rotation_matrix
 
-
 # ############################################
 # Accessors and public properties of the class
 # ############################################
@@ -2081,9 +2167,6 @@ class Geometry(object):
         self._cached_array["q_corner"] = q_corner
 
     def get_wavelength(self):
-        if self._wavelength is None:
-            raise RuntimeWarning("Using wavelength without having defined"
-                                 " it previously ... excepted to fail !")
         return self._wavelength
 
     wavelength = property(get_wavelength, set_wavelength)
@@ -2096,6 +2179,7 @@ class Geometry(object):
 
     def del_ttha(self):
         self._cached_array["2th_center"] = None
+
     ttha = property(get_ttha, set_ttha, del_ttha, "2theta array in cache")
 
     def get_chia(self):
@@ -2106,6 +2190,7 @@ class Geometry(object):
 
     def del_chia(self):
         self._cached_array["chi_center"] = None
+
     chia = property(get_chia, set_chia, del_chia, "chi array in cache")
 
     def get_dssa(self):
@@ -2129,6 +2214,7 @@ class Geometry(object):
 
     def del_qa(self):
         self._cached_array["q_center"] = None
+
     qa = property(get_qa, set_qa, del_qa, "Q array in cache")
 
     def get_ra(self):
@@ -2139,6 +2225,7 @@ class Geometry(object):
 
     def del_ra(self):
         self.self._cached_array["r_center"] = None
+
     ra = property(get_ra, set_ra, del_ra, "R array in cache")
 
     def get_pixel1(self):
