@@ -27,10 +27,13 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
+"""
+Deprecated ... restore or delete !
+"""
 
 __authors__ = ["Jérôme Kieffer", "Giannis Ashiotis"]
 __license__ = "MIT"
-__date__ = "04/10/2018"
+__date__ = "20/01/2021"
 __copyright__ = "2014, ESRF, Grenoble"
 __contact__ = "jerome.kieffer@esrf.fr"
 
@@ -38,19 +41,18 @@ import gc
 import logging
 import threading
 import numpy
-from . import ocl, pyopencl, allocate_cl_buffers, release_cl_buffers
-from ..ext.splitBBoxLUT import HistoBBox1d
-from . import utils
-from ..utils import crc32
-if pyopencl:
+from . import ocl
+if ocl is not None:
+    from . import pyopencl, allocate_cl_buffers, release_cl_buffers
     mf = pyopencl.mem_flags
-else:
-    raise ImportError("pyopencl is not installed")
+    from . import concatenate_cl_kernel
+from ..utils import crc32
 
 logger = logging.getLogger(__name__)
 
 
 class OCL_Hist_Pixelsplit(object):
+
     def __init__(self, pos, bins, image_size, pos0Range=None, pos1Range=None, devicetype="all",
                  padded=False, block_size=32,
                  platformid=None, deviceid=None,
@@ -79,14 +81,14 @@ class OCL_Hist_Pixelsplit(object):
             raise RuntimeError("pos.size != 8 * image_size")
         self.pos0Range = numpy.zeros(1, pyopencl.array.vec.float2)
         self.pos1Range = numpy.zeros(1, pyopencl.array.vec.float2)
-        if (pos0Range is not None) and (len(pos0Range) is 2):
+        if (pos0Range is not None) and (len(pos0Range) == 2):
             self.pos0Range[0][0] = min(pos0Range)
             self.pos0Range[0][1] = max(pos0Range)
         else:
             self.pos0Range[0][0] = -float("inf")
             self.pos0Range[0][1] = +float("inf")
 
-        if (pos1Range is not None) and (len(pos1Range) is 2):
+        if (pos1Range is not None) and (len(pos1Range) == 2):
             self.pos1Range[0][0] = min(pos1Range)
             self.pos1Range[0][1] = max(pos1Range)
         else:
@@ -137,10 +139,10 @@ class OCL_Hist_Pixelsplit(object):
         result = numpy.ndarray(4, dtype=numpy.float32)
         pyopencl.enqueue_copy(self._queue, result, self._cl_mem["minmax"])
         print(result)
-        min0 = pos[:, :, 0].min()
-        max0 = pos[:, :, 0].max()
-        min1 = pos[:, :, 1].min()
-        max1 = pos[:, :, 1].max()
+        min0 = pos[:,:, 0].min()
+        max0 = pos[:,:, 0].max()
+        min1 = pos[:,:, 1].min()
+        max1 = pos[:,:, 1].max()
         minmax = (min0, max0, min1, max1)
 
         print(minmax)
@@ -201,7 +203,7 @@ class OCL_Hist_Pixelsplit(object):
         :param kernel_file: path tothe
         """
         kernel_file = kernel_file or "ocl_hist_pixelsplit.cl"
-        kernel_src = utils.concatenate_cl_kernel([kernel_file])
+        kernel_src = concatenate_cl_kernel([kernel_file])
 
         template_options = "-D BINS=%i  -D NIMAGE=%i -D WORKGROUP_SIZE=%i -D EPS=%f"
         compile_options = template_options % (self.bins, self.size, self.BLOCK_SIZE, numpy.finfo(numpy.float32).eps)
