@@ -31,11 +31,9 @@ separation on GPU.
 
 """
 
-from __future__ import absolute_import, print_function, division
-
 __author__ = "Jérôme Kieffer"
 __license__ = "MIT"
-__date__ = "04/10/2018"
+__date__ = "16/10/2020"
 __copyright__ = "2015, ESRF, Grenoble"
 __contact__ = "jerome.kieffer@esrf.fr"
 
@@ -44,7 +42,7 @@ import logging
 logger = logging.getLogger(__name__)
 from collections import OrderedDict
 import numpy
-from . import ocl, release_cl_buffers, kernel_workgroup_size
+from . import ocl
 if ocl:
     import pyopencl.array
     from . import processing
@@ -53,6 +51,7 @@ if ocl:
     BufferDescription = processing.BufferDescription
 else:
     raise ImportError("pyopencl is not installed or no device is available")
+from. import release_cl_buffers, kernel_workgroup_size, get_x87_volatile_option
 
 
 class Separator(OpenclProcessing):
@@ -81,7 +80,13 @@ class Separator(OpenclProcessing):
         self.npt_height = npt_height
 
         self.allocate_buffers()
-        self.compile_kernels()
+
+        try:
+            default_compiler_options = self.get_compiler_options(x87_volatile=True)
+        except AttributeError:  # Silx version too old
+            logger.warning("Please upgrade to silx v0.10+")
+            default_compiler_options = get_x87_volatile_option(self.ctx)
+        self.compile_kernels(compile_options=default_compiler_options)
         if block_size is None:
             self.block_size = kernel_workgroup_size(self.program, "filter_vertical")
         else:
