@@ -28,13 +28,11 @@
 
 """test suite for Azimuthal integrator class"""
 
-from __future__ import absolute_import, division, print_function
-
 __author__ = "Jérôme Kieffer"
 __contact__ = "Jerome.Kieffer@ESRF.eu"
 __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
-__date__ = "05/12/2018"
+__date__ = "13/01/2021"
 
 import unittest
 import os
@@ -53,15 +51,15 @@ from ..azimuthalIntegrator import AzimuthalIntegrator
 from ..detectors import Detector
 if logger.getEffectiveLevel() <= logging.DEBUG:
     import pylab
-from pyFAI import units
+from pyFAI import units, detector_factory
 from ..utils import mathutil
-from ..third_party import six
 from pyFAI.utils.decorators import depreclog
 
 
 @unittest.skipIf(UtilsTest.low_mem, "test using >500M")
 class TestAzimPilatus(unittest.TestCase):
     """This test uses a lot of memory"""
+
     @classmethod
     def setUpClass(cls):
         cls.img = UtilsTest.getimage("Pilatus6M.cbf")
@@ -158,11 +156,12 @@ class TestAzimHalfFrelon(unittest.TestCase):
         """
         Compare numpy histogram with results of fit2d
         """
-        tth, I = self.ai.integrate1d(self.data,
-                                     len(self.fit2d),
-                                     filename=self.tmpfiles["numpy"],
-                                     correctSolidAngle=False,
-                                     unit="2th_deg")
+        tth, I = self.ai.integrate1d_ng(self.data,
+                                        len(self.fit2d),
+                                        filename=self.tmpfiles["numpy"],
+                                        correctSolidAngle=False,
+                                        method=("no", "histogram", "cython"),
+                                        unit="2th_deg")
         rwp = mathutil.rwp((tth, I), self.fit2d.T)
         logger.info("Rwp numpy/fit2d = %.3f", rwp)
         if logger.getEffectiveLevel() == logging.DEBUG:
@@ -175,7 +174,7 @@ class TestAzimHalfFrelon(unittest.TestCase):
             handles, labels = sp.get_legend_handles_labels()
             fig.legend(handles, labels)
             fig.show()
-            six.moves.input("Press enter to quit")
+            input("Press enter to quit")
         self.assertLess(rwp, 11, "Rwp numpy/fit2d: %.3f" % rwp)
 
     @unittest.skipIf(UtilsTest.low_mem, "test using >100Mb")
@@ -183,14 +182,12 @@ class TestAzimHalfFrelon(unittest.TestCase):
         """
         Compare cython histogram with results of fit2d
         """
-        tth, I = self.ai.integrate1d(self.data,
+        tth, I = self.ai.integrate1d_ng(self.data,
                                      len(self.fit2d),
                                      filename=self.tmpfiles["cython"],
                                      correctSolidAngle=False,
                                      unit='2th_deg',
-                                     method="cython")
-        # logger.info(tth)
-        # logger.info(I)
+                                     method=("no", "histogram", "cython"))
         rwp = mathutil.rwp((tth, I), self.fit2d.T)
         logger.info("Rwp cython/fit2d = %.3f", rwp)
         if logger.getEffectiveLevel() == logging.DEBUG:
@@ -203,7 +200,7 @@ class TestAzimHalfFrelon(unittest.TestCase):
             handles, labels = sp.get_legend_handles_labels()
             fig.legend(handles, labels)
             fig.show()
-            six.moves.input("Press enter to quit")
+            input("Press enter to quit")
         self.assertLess(rwp, 11, "Rwp cython/fit2d: %.3f" % rwp)
 
     @unittest.skipIf(UtilsTest.low_mem, "test using >200M")
@@ -214,17 +211,17 @@ class TestAzimHalfFrelon(unittest.TestCase):
         logger.info(self.ai.__repr__())
         self.ai.corner_array(self.data.shape, unit=units.TTH_RAD, scale=False)
         # this was just to enforce the initalization of the array
-        t0 = time.time()
+        t0 = time.perf_counter()
         logger.info("in test_cythonSP_vs_fit2d Before SP")
 
-        tth, I = self.ai.integrate1d(self.data,
-                                     len(self.fit2d),
-                                     filename=self.tmpfiles["cythonSP"],
-                                     method="splitpixel",
-                                     correctSolidAngle=False,
-                                     unit="2th_deg")
+        tth, I = self.ai.integrate1d_ng(self.data,
+                                        len(self.fit2d),
+                                        filename=self.tmpfiles["cythonSP"],
+                                        method=("full", "histogram", "cython"),
+                                        correctSolidAngle=False,
+                                        unit="2th_deg")
         logger.info("in test_cythonSP_vs_fit2d Before")
-        t1 = time.time() - t0
+        t1 = time.perf_counter() - t0
         rwp = mathutil.rwp((tth, I), self.fit2d.T)
         logger.info("Rwp cythonSP(t=%.3fs)/fit2d = %.3f", t1, rwp)
         if logger.getEffectiveLevel() == logging.DEBUG:
@@ -237,7 +234,7 @@ class TestAzimHalfFrelon(unittest.TestCase):
             handles, labels = sp.get_legend_handles_labels()
             fig.legend(handles, labels)
             fig.show()
-            six.moves.input("Press enter to quit")
+            input("Press enter to quit")
         self.assertLess(rwp, 11, "Rwp cythonSP/fit2d: %.3f" % rwp)
 
     @unittest.skipIf(UtilsTest.low_mem, "test using >100Mb")
@@ -245,25 +242,24 @@ class TestAzimHalfFrelon(unittest.TestCase):
         """
         Compare cython histogram with numpy histogram
         """
-        tth_np, I_np = self.ai.integrate1d(self.__class__.data,
-                                           len(self.fit2d),
-                                           correctSolidAngle=False,
-                                           unit="2th_deg",
-                                           method="numpy")
-        tth_cy, I_cy = self.ai.integrate1d(self.__class__.data,
-                                           len(self.fit2d),
-                                           correctSolidAngle=False,
-                                           unit="2th_deg",
-                                           method="cython")
-        logger.info("before xrpd_splitPixel")
-        tth_sp, I_sp = self.ai.integrate1d(self.__class__.data,
-                                           len(self.fit2d),
-                                           correctSolidAngle=False,
-                                           unit="2th_deg",
-                                           method="splitpixel")
-        logger.info("After xrpd_splitPixel")
+        tth_np, I_np = self.ai.integrate1d_ng(self.__class__.data,
+                                              len(self.fit2d),
+                                              correctSolidAngle=False,
+                                              unit="2th_deg",
+                                              method=("no", "histogram", "numpy"))
+        tth_cy, I_cy = self.ai.integrate1d_ng(self.__class__.data,
+                                              len(self.fit2d),
+                                              correctSolidAngle=False,
+                                              unit="2th_deg",
+                                              method=("no", "histogram", "cython"))
+        tth_sp, I_sp = self.ai.integrate1d_ng(self.__class__.data,
+                                              len(self.fit2d),
+                                              correctSolidAngle=False,
+                                              unit="2th_deg",
+                                              method=("full", "histogram", "cython"))
+
         rwp = mathutil.rwp((tth_cy, I_cy), (tth_np, I_np))
-        logger.info("Rwp = %.3f", rwp)
+        logger.info("Histogram Cython/Numpy Rwp = %.3f", rwp)
         if logger.getEffectiveLevel() == logging.DEBUG:
             logging.info("Plotting results")
             fig = pylab.figure()
@@ -276,16 +272,20 @@ class TestAzimHalfFrelon(unittest.TestCase):
             handles, labels = sp.get_legend_handles_labels()
             fig.legend(handles, labels)
             fig.show()
-            six.moves.input("Press enter to quit")
+            input("Press enter to quit")
 
         self.assertLess(rwp, 3, "Rwp cython/numpy: %.3f" % rwp)
 
     def test_separate(self):
         "test separate with a mask. issue #209 regression test"
         msk = self.data < 100
-        bragg, amorphous = self.ai.separate(self.data, mask=msk)
-        self.assertTrue(amorphous.max() < bragg.max(), "bragg is more intense than amorphous")
-        self.assertTrue(amorphous.std() < bragg.std(), "bragg is more variatic than amorphous")
+        res = self.ai.separate(self.data, mask=msk)
+        bragg, amorphous = res
+
+        self.assertLess(amorphous.max(), bragg.max(), "bragg is more intense than amorphous")
+        self.assertLess(amorphous.std(), bragg.std(), "bragg is more variatic than amorphous")
+        self.assertGreater(numpy.diff(res.radial).min(), 0, "radial position is stricly monotonic")
+        self.assertEqual(res.radial.shape, res.intensity.shape, "1D intensities are of proper shape")
 
     @unittest.skipIf(UtilsTest.opencl is False, "User request to skip OpenCL tests")
     @unittest.skipIf(UtilsTest.low_mem, "test using >100Mb")
@@ -294,13 +294,15 @@ class TestAzimHalfFrelon(unittest.TestCase):
         ocl = self.ai.medfilt1d(self.data, 1000, unit="2th_deg", method="bbox_ocl_csr")
         rwp = mathutil.rwp(ref, ocl)
         logger.info("test_medfilt1d median Rwp = %.3f", rwp)
-        self.assertLess(rwp, 1, "Rwp medfilt1d Numpy/OpenCL: %.3f" % rwp)
+#        print("cython", ref.intensity.min(), ref.intensity.max(), ref.intensity.mean(), ref.intensity.std())
+#        print("opencl", ocl.intensity.min(), ocl.intensity.max(), ocl.intensity.mean(), ocl.intensity.std())
+        self.assertLess(rwp, 1, "Rwp medfilt1d Cython/OpenCL: %.3f" % rwp)
 
         ref = self.ai.medfilt1d(self.data, 1000, unit="2th_deg", method="bbox_csr", percentile=(20, 80))
         ocl = self.ai.medfilt1d(self.data, 1000, unit="2th_deg", method="bbox_ocl_csr", percentile=(20, 80))
         rwp = mathutil.rwp(ref, ocl)
         logger.info("test_medfilt1d trimmed-mean Rwp = %.3f", rwp)
-        self.assertLess(rwp, 3, "Rwp trimmed-mean Numpy/OpenCL: %.3f" % rwp)
+        self.assertLess(rwp, 3, "Rwp trimmed-mean Cython/OpenCL: %.3f" % rwp)
         ref = ocl = rwp = None
         gc.collect()
 
@@ -338,7 +340,7 @@ class TestFlatimage(unittest.TestCase):
             sp = fig.add_subplot(111)
             sp.imshow(I, interpolation="nearest")
             fig.show()
-            six.moves.input("Press enter to quit")
+            input("Press enter to quit")
         I[I == -1.0] = 1.0
         assert abs(I.min() - 1.0) < self.epsilon
         assert abs(I.max() - 1.0) < self.epsilon
@@ -358,7 +360,7 @@ class TestFlatimage(unittest.TestCase):
             sp = fig.add_subplot(111)
             sp.imshow(I, interpolation="nearest")
             fig.show()
-            six.moves.input("Press enter to quit")
+            input("Press enter to quit")
         I[I == -1.0] = 1.0
         assert abs(I.min() - 1.0) < self.epsilon
         assert abs(I.max() - 1.0) < self.epsilon
@@ -445,11 +447,11 @@ class TestSaxs(unittest.TestCase):
         ref1d = {}
         ref2d = {}
 
-        data = fabio.open(self.edfPilatus).data[:ai.detector.shape[0], :ai.detector.shape[1]]
+        data = fabio.open(self.edfPilatus).data[:ai.detector.shape[0],:ai.detector.shape[1]]
         for method in methods:
             logger.debug("TestSaxs.test_normalization_factor method= " + method)
-            ref1d[method + "_1"] = ai.integrate1d(copy.deepcopy(data), 100, method=method, error_model="poisson")
-            ref1d[method + "_10"] = ai.integrate1d(copy.deepcopy(data), 100, method=method, normalization_factor=10, error_model="poisson")
+            ref1d[method + "_1"] = ai.integrate1d_ng(copy.deepcopy(data), 100, method=method, error_model="poisson")
+            ref1d[method + "_10"] = ai.integrate1d_ng(copy.deepcopy(data), 100, method=method, normalization_factor=10, error_model="poisson")
             ratio_i = ref1d[method + "_1"].intensity.mean() / ref1d[method + "_10"].intensity.mean()
             ratio_s = ref1d[method + "_1"].sigma.mean() / ref1d[method + "_10"].sigma.mean()
 
@@ -483,13 +485,15 @@ class TestSaxs(unittest.TestCase):
         ai.setFit2D(2000, 870, 102.123456789)  # rational numbers are hell !
         ai.wavelength = 1e-10
         mask = img < 0
-        res_poisson = ai.integrate1d(img, 1000, mask=mask, error_model="poisson")
+        res_poisson = ai.integrate1d_ng(img, 1000, mask=mask, error_model="poisson")
         self.assertGreater(res_poisson.sigma.min(), 0, "Poisson error are positive")
-        res_azimuthal = ai.integrate1d(img, 1000, mask=mask, error_model="azimuthal")
+# TODO bug 1446 error-model "azimuthal" is not implemented in `integrate1d_ng`
+        res_azimuthal = ai.integrate1d_legacy(img, 1000, mask=mask, error_model="azimuthal")
         self.assertGreater(res_azimuthal.sigma.min(), 0, "Azimuthal error are positive")
 
 
 class TestSetter(unittest.TestCase):
+
     def setUp(self):
         self.ai = AzimuthalIntegrator()
         shape = (10, 15)
@@ -522,6 +526,48 @@ class TestSetter(unittest.TestCase):
         self.assertTrue(abs(self.ai.darkcurrent - 0.5 * (self.rnd1 + self.rnd2)).max() == 0, "Dark array is OK")
 
 
+class TestIntergrationNextGeneration(unittest.TestCase):
+
+    def test_histo(self):
+        det = detector_factory("Pilatus100k")
+        data = numpy.random.random(det.shape)
+        ai = AzimuthalIntegrator(detector=det, wavelength=1e-10)
+
+        method = ("no", "histogram", "python")
+        python = ai._integrate1d_ng(data, 100, method=method, error_model="poisson")
+        self.assertEqual(python.compute_engine, "pyFAI.engines.histogram_engine.histogram1d_engine")
+        self.assertEqual(str(python.unit), "q_nm^-1")
+
+        method = ("no", "histogram", "cython")
+        cython = ai._integrate1d_ng(data, 100, method=method, error_model="poisson")
+        self.assertEqual(cython.compute_engine, "pyFAI.ext.histogram.histogram1d_engine")
+        self.assertEqual(str(cython.unit), "q_nm^-1")
+        self.assertTrue(numpy.allclose(cython.radial, python.radial), "cython position are the same")
+        self.assertTrue(numpy.allclose(cython.intensity, python.intensity), "cython intensities are the same")
+        self.assertTrue(numpy.allclose(cython.sigma, python.sigma), "cython errors are the same")
+        self.assertTrue(numpy.allclose(cython.sum_signal, python.sum_signal), "cython sum_signal are the same")
+        self.assertTrue(numpy.allclose(cython.sum_variance, python.sum_variance), "cython sum_variance are the same")
+        self.assertTrue(numpy.allclose(cython.sum_normalization, python.sum_normalization), "cython sum_normalization are the same")
+        self.assertTrue(numpy.allclose(cython.count, python.count), "cython count are the same")
+
+        method = ("no", "histogram", "opencl")
+        actual_method = ai._normalize_method(method=method, dim=1, default=ai.DEFAULT_METHOD_1D).method[1:4]
+        if actual_method != method:
+            reason = "Skipping TestIntergrationNextGeneration.test_histo as OpenCL method not available"
+            self.skipTest(reason)
+        opencl = ai._integrate1d_ng(data, 100, method=method, error_model="poisson")
+        self.assertEqual(opencl.compute_engine, "pyFAI.opencl.azim_hist.OCL_Histogram1d")
+        self.assertEqual(str(opencl.unit), "q_nm^-1")
+
+        self.assertTrue(numpy.allclose(opencl.radial, python.radial), "opencl position are the same")
+        self.assertTrue(numpy.allclose(opencl.intensity, python.intensity), "opencl intensities are the same")
+        self.assertTrue(numpy.allclose(opencl.sigma, python.sigma), "opencl errors are the same")
+        self.assertTrue(numpy.allclose(opencl.sum_signal.sum(axis=-1), python.sum_signal), "opencl sum_signal are the same")
+        self.assertTrue(numpy.allclose(opencl.sum_variance.sum(axis=-1), python.sum_variance), "opencl sum_variance are the same")
+        self.assertTrue(numpy.allclose(opencl.sum_normalization.sum(axis=-1), python.sum_normalization), "opencl sum_normalization are the same")
+        self.assertTrue(numpy.allclose(opencl.count, python.count), "opencl count are the same")
+
+
 def suite():
     loader = unittest.defaultTestLoader.loadTestsFromTestCase
     testsuite = unittest.TestSuite()
@@ -531,6 +577,7 @@ def suite():
     # Consumes a lot of memory
     # testsuite.addTest(loader(TestAzimPilatus))
     testsuite.addTest(loader(TestSaxs))
+    testsuite.addTest(loader(TestIntergrationNextGeneration))
     return testsuite
 
 
