@@ -4,7 +4,7 @@
 #    Project: Azimuthal integration
 #             https://github.com/silx-kit/pyFAI
 #
-#    Copyright (C) 2015-2018 European Synchrotron Radiation Facility, Grenoble, France
+#    Copyright (C) 2015-2020 European Synchrotron Radiation Facility, Grenoble, France
 #
 #    Principal author:       Jérôme Kieffer (Jerome.Kieffer@ESRF.eu)
 #
@@ -78,14 +78,11 @@ Here are the valid keys:
 - "method"
 """
 
-
-from __future__ import with_statement, print_function, division
-
 __author__ = "Jerome Kieffer"
 __contact__ = "Jerome.Kieffer@ESRF.eu"
 __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
-__date__ = "17/12/2018"
+__date__ = "14/01/2021"
 __status__ = "development"
 
 import threading
@@ -93,16 +90,26 @@ import os.path
 import logging
 import json
 import numpy
-import fabio
+from collections import OrderedDict
 
 logger = logging.getLogger(__name__)
 
 from . import average
-from .detectors import detector_factory
+from . import method_registry
 from .azimuthalIntegrator import AzimuthalIntegrator
 from .distortion import Distortion
 from . import units
+from .io import integration_config
+import pyFAI.io.image
 from .engines.preproc import preproc as preproc_numpy
+try:
+    import numexpr
+except ImportError as err:
+    logger.warning("Unable to import Cython version of preproc: %s", err)
+    USE_NUMEXPR = False
+else:
+    USE_NUMEXPR = True
+
 try:
     from .ext.preproc import preproc
 except ImportError as err:
@@ -121,6 +128,7 @@ def make_ai(config, consume_keys=False):
         consumed when used.
     :return: A configured (but uninitialized) :class:`AzimuthalIntgrator`.
     """
+    config = integration_config.normalize(config, inplace=consume_keys)
     ai = AzimuthalIntegrator()
     _init_ai(ai, config, consume_keys)
     return ai
@@ -139,59 +147,10 @@ def _init_ai(ai, config, consume_keys=False, read_maps=True):
     if not consume_keys:
         config = dict(config)
 
-    # Poni-file
-    # NOTE: Compatibility (poni is not stored since pyFAI v0.17)
-    value = config.pop("poni", None)
-    if value:
-        if not os.path.exists(value):
-            logger.warning("Poni-file '%s' not found. The axymuthal integrator is maybe to created as expected", value)
-        else:
-            ai.load(value)
-
-    # Geometry
-    for key in ("dist", "poni1", "poni2", "rot1", "rot2", "rot3"):
-        value = config.pop(key, None)
-        if value is not None:
-            ai.__setattr__(key, value)
-    wavelength = config.pop("wavelength", None)
-    if wavelength:
-        if wavelength <= 0 or wavelength > 1e-6:
-            logger.warning("Wavelength is in meter... unlikely value %s", wavelength)
-        ai.wavelength = wavelength
-
-    # Detector
-    value = config.pop("detector_config", None)
-    if value:
-        # NOTE: Default way to describe a detector since pyFAI 0.17
-        detector_config = value
-        detector_class = config.pop("detector")
-        detector = detector_factory(detector_class, config=detector_config)
-        ai.detector = detector
-    else:
-        value = config.pop("detector", None)
-        if value:
-            # NOTE: Previous way to describe a detector before pyFAI 0.17
-            # NOTE: pixel1/pixel2/splineFile was not parsed here
-            detector_name = value.lower()
-            detector = detector_factory(detector_name)
-
-            if detector_name == "detector":
-                value = config.pop("pixel1", None)
-                if value:
-                    detector.set_pixel1(value)
-                value = config.pop("pixel2", None)
-                if value:
-                    detector.set_pixel2(value)
-            else:
-                # Drop it as it was not really used
-                _ = config.pop("pixel1", None)
-                _ = config.pop("pixel2", None)
-
-            splineFile = config.pop("splineFile", None)
-            if splineFile:
-                detector.set_splineFile(splineFile)
-
-            ai.detector = detector
+#   #This sets only what is part of the poni-file
+    config_reader = integration_config.ConfigurationReader(config)
+    poni = config_reader.pop_ponifile()
+    ai._init_from_poni(poni)
 
     value = config.pop("chi_discontinuity_at_0", False)
     if value:
@@ -202,41 +161,39 @@ def _init_ai(ai, config, consume_keys=False, read_maps=True):
     if read_maps:
         filename = config.pop("mask_file", "")
         apply_process = config.pop("do_mask", True)
-        if filename and os.path.exists(filename) and apply_process:
+        if filename and apply_process:
             try:
-                data = fabio.open(filename).data
+                data = pyFAI.io.image.read_image_data(filename)
             except Exception as error:
                 logger.error("Unable to load mask file %s, error %s", filename, error)
             else:
-                ai.mask = data
+                ai.detector.mask = data
 
         filename = config.pop("dark_current", "")
         apply_process = config.pop("do_dark", True)
         if filename and apply_process:
-            filenames = _read_filenames(filename)
-            ai.set_darkfiles(filenames)
+            ai.detector.set_darkcurrent(_reduce_images(_normalize_filenames(filename)))
 
         filename = config.pop("flat_field", "")
         apply_process = config.pop("do_flat", True)
         if filename and apply_process:
-            filenames = _read_filenames(filename)
-            ai.set_flatfiles(filenames)
-
+            ai.detector.set_flatfield(_reduce_images(_normalize_filenames(filename)))
     return ai
 
 
-def _read_filenames(filenames):
-    """Returns a list of strings from a comma separated string list or a list.
+def _normalize_filenames(filenames):
+    """Returns a list of strings from a string or a list of strings.
 
     :rtype: List[str]
     """
-    if filenames is None:
+    if filenames is None or filenames == "":
         return []
     if isinstance(filenames, list):
         return filenames
-    if "," in filenames:
-        logger.warning("Dark or flat files are described using comma separator list. You should use a python/json list of string instead.")
-    return filenames.split(",")
+    if isinstance(filenames, (str,)):
+        # It's a single filename
+        return [filenames]
+    raise TypeError("Unsupported type %s for a list of filenames" % type(filenames))
 
 
 def _reduce_images(filenames, method="mean"):
@@ -248,50 +205,28 @@ def _reduce_images(filenames, method="mean"):
     """
     if len(filenames) == 0:
         return None
-    if fabio is None:
-        raise RuntimeError("FabIO is missing")
     if len(filenames) == 1:
-        return fabio.open(filenames[0]).data.astype(numpy.float32)
+        return pyFAI.io.image.read_image_data(filenames[0]).astype(numpy.float32)
     else:
         return average.average_images(filenames, filter_=method, fformat=None, threshold=0)
 
 
 class Worker(object):
+
     def __init__(self, azimuthalIntegrator=None,
                  shapeIn=(2048, 2048), shapeOut=(360, 500),
-                 unit="r_mm", dummy=None, delta_dummy=None,
-                 azimuthalIntgrator=None):
+                 unit="r_mm", dummy=None, delta_dummy=None):
         """
         :param AzimuthalIntegrator azimuthalIntegrator: An AzimuthalIntegrator instance
-        :param AzimuthalIntegrator azimuthalIntgrator: An AzimuthalIntegrator instance (deprecated)
         :param shapeIn: image size in input
         :param shapeOut: Integrated size: can be (1,2000) for 1D integration
         :param unit: can be "2th_deg, r_mm or q_nm^-1 ...
         """
-        # TODO remove it in few month (added on 2016-08-04)
-        if azimuthalIntgrator is not None:
-            logger.warning("'Worker(azimuthalIntgrator=...)' parameter is deprecated cause it contains a typo. Please use 'azimuthalIntegrator='")
-            azimuthalIntegrator = azimuthalIntgrator
-
         self._sem = threading.Semaphore()
         if azimuthalIntegrator is None:
             self.ai = AzimuthalIntegrator()
         else:
             self.ai = azimuthalIntegrator
-#        self.config = {}
-#        self.config_file = "azimInt.json"
-#        self.nbpt_azim = 0
-#        if type(config) == dict:
-#            self.config = config
-#        elif type(config) in types.StringTypes:
-#            if os.path.isfile(config):
-#                with open(config, "r") as f:
-#                    self.config = json.load(f)
-#                self.config_file(config)
-#            else:
-#                self.config = json.loads(config)
-#        if self.config:
-#            self.configure()
         self._normalization_factor = None  # Value of the monitor: divides the intensity by this value for normalization
         self.nbpt_azim, self.nbpt_rad = shapeOut
         self._unit = units.to_unit(unit)
@@ -377,7 +312,6 @@ class Worker(object):
                  "method": self.method,
                  "polarization_factor": self.polarization_factor,
                  # "filename": None,
-                 "safe": self.safe,
                  "data": data,
                  "correctSolidAngle": self.correct_solid_angle,
                  "safe": self.safe,
@@ -393,18 +327,8 @@ class Worker(object):
         if self.do_2D():
             kwarg["npt_rad"] = self.nbpt_rad
             kwarg["npt_azim"] = self.nbpt_azim
-            # if "filename" in kwarg:
-            #    if self.extension:
-            #        kwarg["filename"] += self.extension
-            #    else:
-            #        kwarg["filename"] += ".azim"
         else:
             kwarg["npt"] = self.nbpt_rad
-            # if "filename" in kwarg:
-            #    if self.extension:
-            #        kwarg["filename"] += self.extension
-            #    else:
-            #        kwarg["filename"] += ".xy"
         kwarg["error_model"] = self.error_model
 
         if self.radial_range is not None:
@@ -423,7 +347,7 @@ class Worker(object):
                 if variance is not None:
                     error = integrated_result.sigma
             else:
-                integrated_result = self.ai.integrate1d(**kwarg)
+                integrated_result = self.ai.integrate1d_ng(**kwarg)
                 self.radial = integrated_result.radial
                 self.azimuthal = None
                 result = numpy.vstack(integrated_result).T
@@ -438,9 +362,7 @@ class Worker(object):
                     "ai:",
                     str(self.ai),
                     "method:",
-                    kwarg.get("method")
-                    # str(self.ai._csr_integrator),
-                    # "csr size: %s" % self.ai._lut_integrator.size
+                    str(kwarg.get("method"))
                     ]
             logger.error("\n".join(err2))
             raise err
@@ -471,21 +393,17 @@ class Worker(object):
         else:
             self.extension = None
 
-    def setDarkcurrentFile(self, imagefile):
-        self.ai.set_darkfiles(imagefile)
+    def set_dark_current_file(self, imagefile):
+        self.ai.detector.set_darkcurrent(_reduce_images(imagefile))
         self.dark_current_image = imagefile
 
-    def setFlatfieldFile(self, imagefile):
-        self.ai.set_flatfiles(imagefile)
+    setDarkcurrentFile = set_dark_current_file
+
+    def set_flat_field_file(self, imagefile):
+        self.ai.detector.set_flatfield(_reduce_images(imagefile))
         self.flat_field_image = imagefile
 
-    def setJsonConfig(self, json_file):
-        if os.path.isfile(json_file):
-            with open(json_file, "r") as f:
-                config = json.load(f)
-        else:
-            config = json.loads(json_file)
-        self.set_config(config)
+    setFlatfieldFile = set_flat_field_file
 
     def set_config(self, config, consume_keys=False):
         """
@@ -499,14 +417,15 @@ class Worker(object):
             # Avoid to edit the input argument
             config = dict(config)
 
+        integration_config.normalize(config, inplace=True)
         _init_ai(self.ai, config, consume_keys=True, read_maps=False)
 
         # Do it here before reading the AI to be able to catch the io
         filename = config.pop("mask_file", "")
         apply_process = config.pop("do_mask", True)
-        if filename and os.path.exists(filename) and apply_process:
+        if filename and apply_process:
             try:
-                data = fabio.open(filename).data
+                data = pyFAI.io.image.read_image_data(filename)
             except Exception as error:
                 logger.error("Unable to load mask file %s, error %s", filename, error)
             else:
@@ -516,7 +435,7 @@ class Worker(object):
         filename = config.pop("dark_current", "")
         apply_process = config.pop("do_dark", True)
         if filename and apply_process:
-            filenames = _read_filenames(filename)
+            filenames = _normalize_filenames(filename)
             method = "mean"
             data = _reduce_images(filenames, method=method)
             self.ai.detector.set_darkcurrent(data)
@@ -526,7 +445,7 @@ class Worker(object):
         filename = config.pop("flat_field", "")
         apply_process = config.pop("do_flat", True)
         if filename and apply_process:
-            filenames = _read_filenames(filename)
+            filenames = _normalize_filenames(filename)
             method = "mean"
             data = _reduce_images(filenames, method=method)
             self.ai.detector.set_flatfield(data)
@@ -539,8 +458,10 @@ class Worker(object):
         else:
             self.nbpt_azim = 1
 
-        do_2D = config.pop("do_2D", False)
-        if do_2D is False:
+        reader = integration_config.ConfigurationReader(config)
+        self.method = reader.pop_method("csr")
+
+        if self.method.dim == 1:
             self.nbpt_azim = 1
 
         value = config.pop("nbpt_rad", None)
@@ -581,17 +502,10 @@ class Worker(object):
         if not apply_values:
             self.dummy, self.delta_dummy = None, None
 
-        do_opencl = config.pop("do_OpenCL", None)
-        method = config.pop("method", None)
-        if do_opencl is not None and method is not None:
-            logger.warning("Both 'method' and 'do_OpenCL' are defined. 'do_OpenCL' is ignored.")
-            do_opencl = None
-        if method is not None:
-            self.method = method
-        elif do_opencl is not None and do_opencl:
-            self.method = "csr_ocl"
-        else:
-            self.method = "csr"
+        self._normalization_factor = config.pop("normalization_factor", None)
+
+        if "monitor_name" in config:
+            logger.warning("Monitor name defined but unsupported by the worker.")
 
         logger.info(self.ai.__repr__())
         self.reset()
@@ -625,29 +539,56 @@ class Worker(object):
 
         FIXME: The returned dictionary is not exhaustive.
         """
-        config = {"unit": str(self.unit)}
+        config = OrderedDict()
+        config["unit"] = str(self.unit)
         for key in ["dist", "poni1", "poni2", "rot1", "rot3", "rot2", "pixel1", "pixel2", "splineFile", "wavelength"]:
             try:
                 config[key] = self.ai.__getattribute__(key)
-            except:
+            except Exception:
                 pass
         for key in ["nbpt_azim", "nbpt_rad", "polarization_factor", "dummy", "delta_dummy",
                     "correct_solid_angle", "dark_current_image", "flat_field_image",
                     "mask_image", "do_poisson", "shape", "method"]:
             try:
                 config[key] = self.__getattribute__(key)
-            except:
+            except Exception:
                 pass
 
+        for key in ["azimuth_range", "radial_range"]:
+            try:
+                value = self.__getattribute__(key)
+            except Exception:
+                pass
+            else:
+                if value is not None:
+                    config["do_" + key] = True
+                    config[key + "_min"] = min(value)
+                    config[key + "_max"] = max(value)
+                else:
+                    config["do_" + key] = False
+
         return config
 
     def get_json_config(self):
         """return configuration as a JSON string"""
-        pass  # TODO
+        return json.dumps(self.get_config(), indent=2)
+
+    def set_json_config(self, json_file):
+        if os.path.isfile(json_file):
+            with open(json_file, "r") as f:
+                config = json.load(f)
+        else:
+            config = json.loads(json_file)
+        self.set_config(config)
+
+    setJsonConfig = set_json_config
 
     def save_config(self, filename=None):
+        """Save the configuration as a JSON file"""
         if not filename:
             filename = self.config_file
+        with open(filename, "w") as w:
+            w.write(self.get_json_config())
 
     def warmup(self, sync=False):
         """
@@ -670,8 +611,28 @@ class Worker(object):
     def set_normalization_factor(self, value):
         with self._sem:
             self._normalization_factor = value
+
     normalization_factor = property(get_normalization_factor, set_normalization_factor)
 
+    def set_method(self, method="csr"):
+        "Set the integration method"
+        dim = 2 if self.do_2D() else 1
+        if method is None:
+            method = method_registry.Method(dim, "*", "*", "*", target=None)
+        elif isinstance(method, method_registry.Method):
+            method = method.fixed(dim=dim)
+        elif isinstance(method, (str,)):
+            method = method_registry.Method.parsed(method)
+            method = method.fixed(dim=dim)
+        elif isinstance(method, (list, tuple)):
+            if len(method) != 3:
+                raise TypeError("Method size %s unsupported." % len(method))
+            split, algo, impl = method
+            method = method_registry.Method(dim, split, algo, impl, target=None)
+        else:
+            raise TypeError("Method type %s unsupported." % type(method))
+        return method
+
     __call__ = process
 
 
@@ -679,6 +640,7 @@ class PixelwiseWorker(object):
     """
     Simple worker doing dark, flat, solid angle and polarization correction
     """
+
     def __init__(self, dark=None, flat=None, solidangle=None, polarization=None,
                  mask=None, dummy=None, delta_dummy=None, device=None,
                  empty=None, dtype="float32"):
@@ -768,15 +730,18 @@ class DistortionWorker(object):
     """
     Simple worker doing dark, flat, solid angle and polarization correction
     """
+
     def __init__(self, detector=None, dark=None, flat=None, solidangle=None, polarization=None,
-                 mask=None, dummy=None, delta_dummy=None, device=None):
+                 mask=None, dummy=None, delta_dummy=None, method="LUT", device=None):
         """Constructor of the worker
         :param dark: array
         :param flat: array
         :param solidangle: solid-angle array
         :param polarization: numpy array with 2D polarization corrections
+        :param dummy: value for bad pixels
+        :param delta_dummy: precision for dummies
+        :param method: LUT or CSR for the correction
         :param device: Used to influance OpenCL behavour: can be "cpu", "GPU", "Acc" or even an OpenCL context
-
         """
 
         self.ctx = None
@@ -799,47 +764,73 @@ class DistortionWorker(object):
 
         if mask is None:
             self.mask = False
+            mask = numpy.zeros(detector.shape, dtype=bool)
         elif mask.min() < 0 and mask.max() == 0:  # 0 is valid, <0 is invalid
-            self.mask = (mask < 0)
+            mask = self.mask = (mask < 0)
         else:
-            self.mask = mask.astype(bool)
+            mask = self.mask = mask.astype(bool)
 
         self.dummy = dummy
         self.delta_dummy = delta_dummy
-        if device is not None:
-            logger.warning("GPU is not yet implemented")
 
-        if detector is None:
-            self.distortion = None
+        if detector is not None:
+            self.distortion = Distortion(detector, method=method, device=device,
+                                     mask=mask, empty=self.dummy or 0)
+            self.distortion.reset(prepare=True)  # enfoce initization
         else:
-            self.distortion = Distortion(detector, method="LUT", device=device,
-                                         mask=self.mask, empty=self.dummy or 0)
+            self.distortion = None
 
-    def process(self, data, variance=None,
+    def process(self,
+                data,
+                variance=None,
                 normalization_factor=1.0):
         """
         Process the data and apply a normalization factor
         :param data: input data
         :param variance: the variance associated to the data
         :param normalization: normalization factor
-        :return: processed data
+        :return: processed data as either an array (data) or two (data, error)
         """
-        proc_data = preproc(data,
-                            variance=variance,
-                            dark=self.dark,
-                            flat=self.flat,
-                            solidangle=self.solidangle,
-                            polarization=self.polarization,
-                            absorption=None,
-                            mask=self.mask,
-                            dummy=self.dummy,
-                            delta_dummy=self.delta_dummy,
-                            normalization_factor=normalization_factor,
-                            empty=None)
-
         if self.distortion is not None:
-            return self.distortion.correct(proc_data, self.dummy, self.delta_dummy)
+            return self.distortion.correct_ng(data,
+                                              variance=variance,
+                                              dark=self.dark,
+                                              flat=self.flat,
+                                              solidangle=self.solidangle,
+                                              polarization=self.polarization,
+                                              dummy=self.dummy,
+                                              delta_dummy=self.delta_dummy,
+                                              normalization_factor=normalization_factor)
         else:
-            return data
+            proc_data = preproc(data,
+                                variance=variance,
+                                dark=self.dark,
+                                flat=self.flat,
+                                solidangle=self.solidangle,
+                                polarization=self.polarization,
+                                absorption=None,
+                                mask=self.mask,
+                                dummy=self.dummy,
+                                delta_dummy=self.delta_dummy,
+                                normalization_factor=normalization_factor,
+                                empty=None)
+            if variance is not None:
+                pp_signal = proc_data[..., 0]
+                pp_variance = proc_data[..., 1]
+                pp_normalisation = proc_data[..., 2]
+                if numexpr is None:
+                    # Cheap, muthithreaded way:
+                    res_signal = numexpr.evaluate("where(pp_normalisation==0.0, 0.0, pp_signal / pp_normalisation)")
+                    res_error = numexpr.evaluate("where(pp_normalisation==0.0, 0.0, sqrt(pp_variance) / abs(pp_normalisation))")
+                else:
+                    # Take the numpy road:
+                    res_signal = numpy.zeros_like(pp_signal)
+                    res_error = numpy.zeros_like(pp_signal)
+                    msk = numpy.where(pp_normalisation != 0)
+                    res_signal[msk] = pp_signal[msk] / pp_normalisation[msk]
+                    res_error[msk] = numpy.sqrt(pp_variance[msk]) / abs(pp_normalisation[msk])
+                return res_signal, res_error
+            else:
+                return proc_data
 
     __call__ = process
